@@ -41,7 +41,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "helpers/nullsafe_str.h"
 #include "parse.h"
 
-static int __rrr_array_clone (
+static int __rrr_array_clone_values (
 		struct rrr_array *target,
 		const struct rrr_array *source,
 		int do_clone_data
@@ -77,7 +77,7 @@ int rrr_array_clone_without_data (
 		struct rrr_array *target,
 		const struct rrr_array *source
 ) {
-	return __rrr_array_clone(target, source, 0);
+	return __rrr_array_clone_values(target, source, 0);
 }
 
 int rrr_array_append_from (
@@ -88,7 +88,7 @@ int rrr_array_append_from (
 
 	struct rrr_array tmp = {0};
 
-	if ((ret = __rrr_array_clone(&tmp, source, 1)) != 0) {
+	if ((ret = __rrr_array_clone_values(&tmp, source, 1)) != 0) {
 		RRR_MSG_0("Could not clone array in rrr_array_append_from\n");
 		goto out;
 	}
@@ -100,19 +100,64 @@ int rrr_array_append_from (
 	return ret;
 }
 
+int rrr_array_clone (
+		struct rrr_array **target,
+		const struct rrr_array *source
+) {
+	int ret = 0;
+
+	struct rrr_array *array = NULL;
+
+	*target = NULL;
+
+	if ((array = rrr_allocate_zero(sizeof(*array))) == NULL) {
+		RRR_MSG_0("Could not allocate memory in rrr_array_clone\n");
+		goto out;
+	}
+
+	if ((ret = rrr_array_append_from (array, source)) != 0) {
+		goto out;
+	}
+
+	*target = array;
+	array = NULL;
+
+	out:
+	if (array != NULL) {
+		rrr_array_destroy(array);
+	}
+	return ret;
+}
+
+#define SET_AND_VERIFY_TAG_LENGTH()                                                 \
+    rrr_length tag_length = 0;                                                      \
+    do {const rrr_biglength tag_length_big = tag != 0 ? strlen(tag) : 0;            \
+    if (tag_length_big > RRR_TYPE_TAG_MAX) {                                        \
+            RRR_MSG_0("Tag was too long when pushing array value (%llu>%llu)\n",    \
+            (unsigned long long) tag_length_big,                                    \
+            (unsigned long long) RRR_TYPE_TAG_MAX                                   \
+        );                                                                          \
+        ret = RRR_ARRAY_SOFT_ERROR;                                                 \
+        goto out;                                                                   \
+    }                                                                               \
+    tag_length = (rrr_length) tag_length_big;                                       \
+    } while(0);
+
 int rrr_array_push_value_vain_with_tag (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag
 ) {
 	struct rrr_type_value *new_value = NULL;
 
 	int ret = 0;
 
+	SET_AND_VERIFY_TAG_LENGTH();
+
 	if ((ret = rrr_type_value_new (
 			&new_value,
 			&rrr_type_definition_vain,
 			0,
-			strlen(tag),
+			tag_length,
 			tag,
 			0,
 			NULL,
@@ -125,7 +170,7 @@ int rrr_array_push_value_vain_with_tag (
 		goto out;
 	}
 
-	RRR_LL_APPEND(collection, new_value);
+	RRR_LL_APPEND(array, new_value);
 	new_value = NULL;
 
 	out:
@@ -136,21 +181,23 @@ int rrr_array_push_value_vain_with_tag (
 }
 
 static int __rrr_array_push_value_64_with_tag (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		uint64_t value,
 		const struct rrr_type_definition *definition,
-		int flags
+		rrr_type_flags flags
 ) {
 	struct rrr_type_value *new_value = NULL;
 
 	int ret = 0;
 
+	SET_AND_VERIFY_TAG_LENGTH();
+
 	if ((ret = rrr_type_value_new (
 			&new_value,
 			definition,
 			flags,
-			strlen(tag),
+			tag_length,
 			tag,
 			sizeof(uint64_t),
 			NULL,
@@ -175,7 +222,7 @@ static int __rrr_array_push_value_64_with_tag (
 		goto out;
 	}
 
-	RRR_LL_APPEND(collection, new_value);
+	RRR_LL_APPEND(array, new_value);
 	new_value = NULL;
 
 	out:
@@ -186,12 +233,12 @@ static int __rrr_array_push_value_64_with_tag (
 }
 
 int rrr_array_push_value_u64_with_tag (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		uint64_t value
 ) {
 	return __rrr_array_push_value_64_with_tag(
-			collection,
+			array,
 			tag,
 			value,
 			&rrr_type_definition_h,
@@ -200,12 +247,12 @@ int rrr_array_push_value_u64_with_tag (
 }
 
 int rrr_array_push_value_i64_with_tag (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		int64_t value
 ) {
 	return __rrr_array_push_value_64_with_tag(
-			collection,
+			array,
 			tag,
 			(uint64_t ) value,
 			&rrr_type_definition_h,
@@ -214,43 +261,49 @@ int rrr_array_push_value_i64_with_tag (
 }
 
 static int __rrr_array_push_value_x_with_tag_with_size (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		const char *value,
-		size_t value_size,
+		rrr_length value_size,
 		const struct rrr_type_definition *type
 ) {
 	struct rrr_type_value *new_value = NULL;
-	if (rrr_type_value_new (
+
+	int ret = 0;
+
+	SET_AND_VERIFY_TAG_LENGTH();
+
+	if ((ret = rrr_type_value_new (
 			&new_value,
 			type,
 			0,
-			tag != NULL ? strlen(tag) : 0,
+			tag_length,
 			tag,
 			value_size,
 			NULL,
 			1,
 			NULL,
 			value_size
-	) != 0) {
+	)) != 0) {
 		RRR_MSG_0("Could not create value in __rrr_array_push_value_x_with_tag_with_size\n");
-		return 1;
+		goto out;
 	}
 
-	RRR_LL_APPEND(collection, new_value);
+	RRR_LL_APPEND(array, new_value);
 
 	memcpy(new_value->data, value, value_size);
 
-	return 0;
+	out:
+	return ret;
 }
 
 int rrr_array_push_value_fixp_with_tag (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		rrr_fixp value
 ) {
 	return __rrr_array_push_value_x_with_tag_with_size (
-			collection,
+			array,
 			tag,
 			(const char *) &value,
 			sizeof(value),
@@ -259,15 +312,15 @@ int rrr_array_push_value_fixp_with_tag (
 }
 
 int rrr_array_push_value_str_with_tag_with_size (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		const char *value,
-		size_t value_size
+		rrr_length value_size
 ) {
 	// Don't use the import function, it reads strings with quotes around it
 
 	return __rrr_array_push_value_x_with_tag_with_size (
-			collection,
+			array,
 			tag,
 			value,
 			value_size,
@@ -276,13 +329,13 @@ int rrr_array_push_value_str_with_tag_with_size (
 }
 
 int rrr_array_push_value_blob_with_tag_with_size (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		const char *value,
-		size_t value_size
+		rrr_length value_size
 ) {
 	return __rrr_array_push_value_x_with_tag_with_size (
-			collection,
+			array,
 			tag,
 			value,
 			value_size,
@@ -291,34 +344,40 @@ int rrr_array_push_value_blob_with_tag_with_size (
 }
 
 struct rrr_array_push_value_blob_with_tag_nullsafe_callback_data {
-	struct rrr_array *collection;
+	struct rrr_array *array;
 	const char *tag;
 	const struct rrr_type_definition *definition;
 };
 
 static int __rrr_array_push_value_x_with_tag_nullsafe_callback (
 		const void *str,
-		rrr_length len,
+		rrr_nullsafe_len len,
 		void *arg
 ) {
 	struct rrr_array_push_value_blob_with_tag_nullsafe_callback_data *callback_data = arg;
 
+	if (len > RRR_LENGTH_MAX) {
+		RRR_MSG_0("Value too long while pushing to array (%" PRIrrr_nullsafe_len ">%llu)\n",
+			len, (unsigned long long) RRR_LENGTH_MAX);
+		return 1;
+	}
+
 	return __rrr_array_push_value_x_with_tag_with_size (
-			callback_data->collection,
+			callback_data->array,
 			callback_data->tag,
 			str,
-			len,
+			(rrr_length) len,
 			callback_data->definition
 	);
 }
 
 int rrr_array_push_value_blob_with_tag_nullsafe (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		const struct rrr_nullsafe_str *str
 ) {
 	struct rrr_array_push_value_blob_with_tag_nullsafe_callback_data callback_data = {
-			collection,
+			array,
 			tag,
 			&rrr_type_definition_blob
 	};
@@ -331,12 +390,12 @@ int rrr_array_push_value_blob_with_tag_nullsafe (
 }
 
 int rrr_array_push_value_str_with_tag_nullsafe (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		const struct rrr_nullsafe_str *str
 ) {
 	struct rrr_array_push_value_blob_with_tag_nullsafe_callback_data callback_data = {
-			collection,
+			array,
 			tag,
 			&rrr_type_definition_str
 	};
@@ -349,87 +408,205 @@ int rrr_array_push_value_str_with_tag_nullsafe (
 }
 
 int rrr_array_push_value_str_with_tag (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const char *tag,
 		const char *value
 ) {
-	size_t value_size = strlen(value);
+	rrr_biglength value_size = strlen(value);
+
+	if (value_size > RRR_LENGTH_MAX) {
+		RRR_MSG_0("Value size too big while pushing string to array (%llu > %llu)\n",
+			(unsigned long long) value_size,
+			(unsigned long long) RRR_LENGTH_MAX
+		);
+	}
 
 	return rrr_array_push_value_str_with_tag_with_size(
-			collection,
+			array,
 			tag,
 			value,
-			value_size
+			(rrr_length) value_size
 	);
 }
 
-int rrr_array_get_value_unsigned_64_by_tag (
-		uint64_t *result,
+static int __rrr_array_get_value_64_by_tag (
+		void *result,
 		struct rrr_array *array,
 		const char *tag,
-		int index
+		unsigned int index,
+		int do_signed
 ) {
 	int ret = 0;
 
-	struct rrr_type_value *value = NULL;
+	const struct rrr_type_value *value = NULL;
 
-	if ((value = rrr_array_value_get_by_tag(array, tag)) == NULL) {
+	if ((value = rrr_array_value_get_by_tag_const(array, tag)) == NULL) {
 		RRR_MSG_0("Could not find value '%s' in array while getting 64-value\n", tag);
 		ret = 1;
 		goto out;
 	}
 
-	if (RRR_TYPE_FLAG_IS_SIGNED(value->flags)) {
+	if (RRR_TYPE_FLAG_IS_SIGNED(value->flags) && !do_signed) {
 		RRR_MSG_0("Value '%s' in array was signed but unsigned value was expected\n", tag);
 		ret = 1;
 		goto out;
 	}
-
-	if (index < 0) {
-		RRR_BUG("Negative index given to rrr_array_get_value_unsigned_64_by_tag\n");
+	else if (!RRR_TYPE_FLAG_IS_SIGNED(value->flags) && do_signed) {
+		RRR_MSG_0("Value '%s' in array was unsigned but signed value was expected\n", tag);
+		ret = 1;
+		goto out;
 	}
 
-	if (index - 1 > (int) value->element_count) {
+	if (index >= value->element_count) {
 		RRR_MSG_0("Array value '%s' index %i was requested but there are only %i elements in the value",
 				tag, index, value->element_count);
 		ret = 1;
 		goto out;
 	}
 
-	*result = *((uint64_t*) value->data + (sizeof(uint64_t) * index));
+	if (do_signed) {
+		*((int64_t *) result) = *((int64_t*) value->data + (sizeof(int64_t) * index));
+	}
+	else {
+		*((uint64_t *) result) = *((uint64_t*) value->data + (sizeof(uint64_t) * index));
+	}
 
 	out:
 	return ret;
 }
 
+int rrr_array_get_value_unsigned_64_by_tag (
+		uint64_t *result,
+		struct rrr_array *array,
+		const char *tag,
+		unsigned int index
+) {
+	return __rrr_array_get_value_64_by_tag (result, array, tag, index, 0 /* Not signed */);
+}
+
+int rrr_array_get_value_signed_64_by_tag (
+		int64_t *result,
+		struct rrr_array *array,
+		const char *tag,
+		unsigned int index
+) {
+	return __rrr_array_get_value_64_by_tag (result, array, tag, index, 1 /* Signed */);
+}
+
+int rrr_array_get_value_str_by_tag (
+		char **result,
+		struct rrr_array *array,
+		const char *tag
+) {
+	int ret = 0;
+
+	const struct rrr_type_value *value = NULL;
+	char *str = NULL;
+
+	if ((value = rrr_array_value_get_by_tag_const(array, tag)) == NULL) {
+		RRR_MSG_0("Could not find value '%s' in array while getting str-value\n", tag);
+		ret = 1;
+		goto out;
+	}
+
+	if (value->definition->to_str == NULL) {
+		RRR_MSG_0("Value '%s' of type '%s' can't be converted to string\n", tag, value->definition->identifier);
+		ret = 1;
+		goto out;
+	}
+
+	if ((ret = value->definition->to_str(&str, value)) != 0) {
+		goto out;
+	}
+
+	*result = str;
+	str = NULL;
+
+	out:
+	RRR_FREE_IF_NOT_NULL(str);
+	return ret;
+}
+
 void rrr_array_strip_type (
-		struct rrr_array *collection,
+		struct rrr_array *array,
 		const struct rrr_type_definition *definition
 ) {
-	RRR_LL_ITERATE_BEGIN(collection, struct rrr_type_value);
+	RRR_LL_ITERATE_BEGIN(array, struct rrr_type_value);
 		if (node->definition == definition) {
 			RRR_LL_ITERATE_SET_DESTROY();
 		}
-	RRR_LL_ITERATE_END_CHECK_DESTROY(collection, 0; rrr_type_value_destroy(node));
+	RRR_LL_ITERATE_END_CHECK_DESTROY(array, 0; rrr_type_value_destroy(node));
 }
 
-void rrr_array_clear (struct rrr_array *collection) {
-	RRR_LL_DESTROY(collection,struct rrr_type_value,rrr_type_value_destroy(node));
+void rrr_array_clear (struct rrr_array *array) {
+	RRR_LL_DESTROY(array,struct rrr_type_value,rrr_type_value_destroy(node));
 }
 
-void rrr_array_clear_void (void *collection) {
-	rrr_array_clear(collection);
+void rrr_array_clear_void (void *array) {
+	rrr_array_clear(array);
 }
 
-void rrr_array_clear_by_tag (struct rrr_array *collection, const char *tag) {
-	RRR_LL_ITERATE_BEGIN(collection, struct rrr_type_value);
+void rrr_array_clear_by_tag_checked (unsigned int *cleared_count, struct rrr_array *array, const char *tag) {
+	*cleared_count = 0;
+
+	RRR_LL_ITERATE_BEGIN(array, struct rrr_type_value);
 		if (node->tag == NULL) {
 			RRR_LL_ITERATE_NEXT();
 		}
 		if (strcmp(node->tag, tag) == 0) {
 			RRR_LL_ITERATE_SET_DESTROY();
+			(*cleared_count)++;
 		}
-	RRR_LL_ITERATE_END_CHECK_DESTROY(collection, 0; rrr_type_value_destroy(node));
+	RRR_LL_ITERATE_END_CHECK_DESTROY(array, 0; rrr_type_value_destroy(node));
+}
+
+void rrr_array_clear_by_tag (struct rrr_array *array, const char *tag) {
+	unsigned int cleared_count_dummy = 0;
+	rrr_array_clear_by_tag_checked(&cleared_count_dummy, array, tag);
+}
+
+void rrr_array_destroy (struct rrr_array *array) {
+	rrr_array_clear(array);
+	rrr_free(array);
+}
+
+void rrr_array_collection_clear (struct rrr_array_collection *collection) {
+	RRR_LL_DESTROY(collection, struct rrr_array, rrr_array_destroy(node));
+}
+
+void rrr_array_trim (struct rrr_array *array, int target_length) {
+	if (target_length > RRR_LL_COUNT(array) || target_length < 0) {
+		RRR_BUG("BUG: Target length was > array length or negative in rrr_array_trim (%i vs %i)\n",
+				target_length, RRR_LL_COUNT(array));
+	}
+
+	// Note : Reverse iteration
+	RRR_LL_ITERATE_BEGIN_REVERSE(array, struct rrr_type_value);
+		if (RRR_LL_COUNT(array) > target_length) {
+			RRR_LL_ITERATE_SET_DESTROY();
+		}
+		else {
+			RRR_LL_ITERATE_LAST();
+		}
+	RRR_LL_ITERATE_END_CHECK_DESTROY(array, 0; rrr_type_value_destroy(node));
+}
+
+void rrr_array_rotate_reverse (struct rrr_array *array) {
+	if (RRR_LL_COUNT(array) == 0) {
+		return;
+	}
+
+	struct rrr_type_value *first = RRR_LL_SHIFT(array);
+	RRR_LL_APPEND(array, first);
+}
+
+void rrr_array_rotate_forward (struct rrr_array *array) {
+	if (RRR_LL_COUNT(array) == 0) {
+		return;
+	}
+
+	struct rrr_type_value *last = RRR_LL_POP(array);
+	RRR_LL_UNSHIFT(array, last);
 }
 
 struct rrr_type_value *rrr_array_value_get_by_index (
@@ -478,32 +655,71 @@ const struct rrr_type_value *rrr_array_value_get_by_tag_const (
 	return NULL;
 }
 
-ssize_t rrr_array_get_packed_length (
-		const struct rrr_array *definition
+int rrr_array_has_tag (
+		const struct rrr_array *definition,
+		const char *tag
 ) {
-	ssize_t result = 0;
-
 	RRR_LL_ITERATE_BEGIN(definition, const struct rrr_type_value);
-		result += node->total_stored_length + sizeof(struct rrr_array_value_packed) - 1;
-		result += node->tag_length;
+		if (node->tag != NULL) {
+			if (strcmp(node->tag, tag) == 0) {
+				return 1;
+			}
+		}
 	RRR_LL_ITERATE_END();
 
-	return result;
+	return 0;
 }
 
-static ssize_t __rrr_array_get_exported_length (
+static int __rrr_array_get_packed_length (
+		rrr_biglength *result,
 		const struct rrr_array *definition
 ) {
-	ssize_t result = 0;
+	rrr_biglength sum = 0;
 
 	RRR_LL_ITERATE_BEGIN(definition, const struct rrr_type_value);
-		result += rrr_type_value_get_export_length(node);
+		rrr_biglength a = node->tag_length + node->total_stored_length + sizeof(struct rrr_array_value_packed) - 1;
+		rrr_biglength b = sum + a;
+		if (b < sum || b < a) {
+			return 1;
+		}
+		sum = b;
 	RRR_LL_ITERATE_END();
 
-	return result;
+	*result = sum;
+
+	return 0;
 }
 
-static int __rrr_array_collection_iterate_chosen_tags (
+static int __rrr_array_get_exported_length (
+		rrr_biglength *result,
+		const struct rrr_array *definition
+) {
+	rrr_biglength sum = 0;
+
+	*result = 0;
+
+	RRR_LL_ITERATE_BEGIN(definition, const struct rrr_type_value);
+		rrr_length a = 0;
+
+		if (rrr_type_value_get_export_length(&a, node) != 0) {
+			RRR_MSG_0("Value too long while getting export length of array\n");
+			return RRR_ARRAY_SOFT_ERROR;
+		}
+
+		rrr_biglength b = sum + a;
+		if (b < sum || b < a) {
+			RRR_MSG_0("Overfow while getting export length of array\n");
+			return RRR_ARRAY_SOFT_ERROR;
+		}
+		sum = b;
+	RRR_LL_ITERATE_END();
+
+	*result = sum;
+
+	return 0;
+}
+
+static int __rrr_array_array_iterate_chosen_tags (
 		int *found_tags,
 		const struct rrr_array *definition,
 		const struct rrr_map *tags,
@@ -540,7 +756,7 @@ static int __rrr_array_collection_iterate_chosen_tags (
 					ret = 0;
 					goto out;
 				}
-				RRR_MSG_0("Error from callback in __rrr_array_collection_iterate_chosen_tags\n");
+				RRR_MSG_0("Error from callback in __rrr_array_array_iterate_chosen_tags\n");
 				ret = 1;
 				goto out;
 			}
@@ -552,11 +768,11 @@ static int __rrr_array_collection_iterate_chosen_tags (
 }
 
 struct pack_callback_data {
-	ssize_t written_bytes_total;
+	rrr_biglength written_bytes_total;
 	char *write_pos;
 };
 
-static int __rrr_array_collection_pack_callback (const struct rrr_type_value *node, void *arg) {
+static int __rrr_array_array_pack_callback (const struct rrr_type_value *node, void *arg) {
 	int ret = 0;
 
 	struct pack_callback_data *data = arg;
@@ -564,7 +780,7 @@ static int __rrr_array_collection_pack_callback (const struct rrr_type_value *no
 	uint8_t type = node->definition->type;
 
 	if (node->definition->pack == NULL) {
-		RRR_BUG("No pack function defined for type %u in __rrr_array_collection_pack_callback\n", type);
+		RRR_BUG("No pack function defined for type %u in __rrr_array_array_pack_callback\n", type);
 	}
 
 	struct rrr_array_value_packed *head = (struct rrr_array_value_packed *) data->write_pos;
@@ -581,7 +797,7 @@ static int __rrr_array_collection_pack_callback (const struct rrr_type_value *no
 	uint8_t new_type = 0;
 	rrr_length written_bytes = 0;
 	if (node->definition->pack(data->write_pos, &written_bytes, &new_type, node) != 0) {
-		RRR_MSG_0("Error while packing data of type %u in __rrr_array_collection_pack_callback\n", node->definition->type);
+		RRR_MSG_0("Error while packing data of type %u in __rrr_array_array_pack_callback\n", node->definition->type);
 		ret = RRR_ARRAY_SOFT_ERROR;
 		goto out;
 	}
@@ -595,7 +811,7 @@ static int __rrr_array_collection_pack_callback (const struct rrr_type_value *no
 	head->total_length = rrr_htobe32(written_bytes);
 
 	if (written_bytes < node->total_stored_length) {
-		RRR_BUG("Size mismatch in __rrr_array_collection_pack_callback, too few bytes written\n");
+		RRR_BUG("Size mismatch in __rrr_array_array_pack_callback, too few bytes written\n");
 	}
 
 	out:
@@ -626,7 +842,7 @@ static int __rrr_array_selected_tags_split_callback (const struct rrr_type_value
 		const rrr_length element_size = node->total_stored_length / node->element_count;
 
 		if (element_size * node->element_count != node->total_stored_length) {
-			RRR_MSG_0("Invalid total store length in array value in __rrr_array_collection_split_callback, not divisible by element size\n");
+			RRR_MSG_0("Invalid total store length in array value in __rrr_array_array_split_callback, not divisible by element size\n");
 			ret = 1;
 			goto out;
 		}
@@ -660,7 +876,7 @@ int rrr_array_selected_tags_split (
 		callback_arg
 	};
 
-	return __rrr_array_collection_iterate_chosen_tags (
+	return __rrr_array_array_iterate_chosen_tags (
 			found_tags,
 			definition,
 			tags,
@@ -669,40 +885,40 @@ int rrr_array_selected_tags_split (
 	);
 }
 
-static int __rrr_array_collection_export_callback (const struct rrr_type_value *node, void *arg) {
+static int __rrr_array_array_export_callback (const struct rrr_type_value *node, void *arg) {
 	int ret = 0;
 
 	struct pack_callback_data *data = arg;
 
 	if (node->definition->export == NULL) {
-		RRR_BUG("No export function defined for type %u in __rrr_array_collection_export_callback\n", node->definition->type);
+		RRR_BUG("No export function defined for type %u in __rrr_array_array_export_callback\n", node->definition->type);
 	}
 
 	rrr_length written_bytes = 0;
 	if (node->definition->export(data->write_pos, &written_bytes, node) != 0) {
-		RRR_MSG_0("Error while exporting data of type %u in __rrr_array_collection_export_callback\n", node->definition->type);
+		RRR_MSG_0("Error while exporting data of type %u in __rrr_array_array_export_callback\n", node->definition->type);
 		ret = RRR_ARRAY_SOFT_ERROR;
 		goto out;
 	}
 	data->write_pos += written_bytes;
 	data->written_bytes_total += written_bytes;
 
-	RRR_DBG_3("array export type %s size %" PRIrrrl " total size %li\n",
+	RRR_DBG_3("array export type %s size %" PRIrrrl " total size %" PRIrrrbl "\n",
 			node->definition->identifier, written_bytes, data->written_bytes_total);
 
 	if (written_bytes < node->total_stored_length) {
-		RRR_BUG("Size mismatch in __rrr_array_collection_export_callback, too few bytes written\n");
+		RRR_BUG("Size mismatch in __rrr_array_array_export_callback, too few bytes written\n");
 	}
 
 	out:
 	return ret;
 }
 
-static int __rrr_array_collection_pack_or_export (
+static int __rrr_array_array_pack_or_export (
 		char *target,
 		int *found_tags,
-		ssize_t *written_bytes_final,
-		ssize_t target_size,
+		rrr_biglength *written_bytes_final,
+		rrr_length target_size,
 		const struct rrr_array *definition,
 		const struct rrr_map *tags,
 		int (*method)(const struct rrr_type_value *node, void *arg)
@@ -716,21 +932,21 @@ static int __rrr_array_collection_pack_or_export (
 
 	callback_data.write_pos = target;
 
-	if ((ret = __rrr_array_collection_iterate_chosen_tags (
+	if ((ret = __rrr_array_array_iterate_chosen_tags (
 			found_tags,
 			definition,
 			tags,
 			method,
 			&callback_data
 	)) != 0) {
-		RRR_MSG_0("Error %i from iterator in __rrr_array_collection_pack_or_export\n", ret);
+		RRR_MSG_0("Error %i from iterator in __rrr_array_array_pack_or_export\n", ret);
 		goto out;
 	}
 
 	*written_bytes_final = callback_data.written_bytes_total;
 
 	if (callback_data.write_pos > target + target_size) {
-		RRR_BUG("Buffer write outside bounds in __rrr_array_collection_pack_or_export\n");
+		RRR_BUG("Buffer write outside bounds in __rrr_array_array_pack_or_export\n");
 	}
 
 	out:
@@ -739,34 +955,48 @@ static int __rrr_array_collection_pack_or_export (
 
 int rrr_array_selected_tags_export (
 		char **target,
-		ssize_t *target_size,
+		rrr_biglength *target_size,
 		int *found_tags,
 		const struct rrr_array *definition,
 		const struct rrr_map *tags
 ) {
 	int ret = 0;
 
+	char *result = NULL;
+
 	// We over-allocate here if not all tags are used
-	rrr_length total_data_length = __rrr_array_get_exported_length(definition);
+	rrr_biglength total_data_length = 0;
+
+	if ((ret = __rrr_array_get_exported_length(&total_data_length, definition)) != 0) {
+		goto out;
+	}
+
+	if (total_data_length > RRR_LENGTH_MAX) {
+		RRR_MSG_0("Export size too long while exporting array (%llu > %llu)\n",
+				(unsigned long long) total_data_length,
+				(unsigned long long) RRR_LENGTH_MAX)
+		;
+		ret = RRR_ARRAY_SOFT_ERROR;
+		goto out;
+	}
 
 	*target = NULL;
 	*target_size = 0;
 
-	char *result = rrr_allocate(total_data_length);
-	if (result == NULL) {
+	if ((result = rrr_allocate(total_data_length)) == NULL) {
 		RRR_MSG_0("Could not allocate memory in rrr_array_selected_tags_to_raw\n");
 		ret = 1;
 		goto out;
 	}
 
-	if ((ret = __rrr_array_collection_pack_or_export (
+	if ((ret = __rrr_array_array_pack_or_export (
 			result,
 			found_tags,
 			target_size,
-			total_data_length,
+			(rrr_length) total_data_length,
 			definition,
 			tags,
-			__rrr_array_collection_export_callback
+			__rrr_array_array_export_callback
 	)) != 0) {
 		RRR_MSG_0("Error while converting array in rrr_array_selected_tags_export return was %i\n", ret);
 		goto out;
@@ -780,7 +1010,7 @@ int rrr_array_selected_tags_export (
 	return ret;
 }
 
-int rrr_array_new_message_from_collection (
+int rrr_array_new_message_from_array (
 		struct rrr_msg_msg **final_message,
 		const struct rrr_array *definition,
 		uint64_t time,
@@ -789,14 +1019,29 @@ int rrr_array_new_message_from_collection (
 ) {
 	int ret = 0;
 
+	struct rrr_msg_msg *message = NULL;
+
 	*final_message = NULL;
 
-	rrr_length total_data_length = rrr_array_get_packed_length(definition);
+	rrr_biglength total_data_length = 0;
 
-	struct rrr_msg_msg *message = rrr_msg_msg_new_array(time, topic_length, total_data_length);
-	if (message == NULL) {
-		RRR_MSG_0("Could not create message for data collection\n");
-		ret = RRR_ARRAY_HARD_ERROR;
+	// Allocation errors here are soft errors, likely to be due to big input data
+
+	if (__rrr_array_get_packed_length(&total_data_length, definition) != 0 ||
+	     total_data_length > RRR_LENGTH_MAX
+	) {
+		RRR_MSG_0("Cannot convery array to message, total data length exceeds maximum (%llu>%llu)\n",
+			(unsigned long long) total_data_length,
+			(unsigned long long) RRR_LENGTH_MAX
+		);
+		ret = RRR_ARRAY_SOFT_ERROR;
+		goto out;
+	}
+
+	if ((message = rrr_msg_msg_new_array(time, topic_length, (rrr_u32) total_data_length)) == NULL) {
+		RRR_MSG_0("Could not create message for data array (size was %llu)\n",
+				(unsigned long long) total_data_length);
+		ret = RRR_ARRAY_SOFT_ERROR;
 		goto out;
 	}
 
@@ -807,25 +1052,25 @@ int rrr_array_new_message_from_collection (
 		memcpy(topic_pos, topic, topic_length);
 	}
 
-	ssize_t written_bytes_total = 0;
+	rrr_biglength written_bytes_total = 0;
 	int found_tags = 0;
 
-	if ((ret = __rrr_array_collection_pack_or_export (
+	if ((ret = __rrr_array_array_pack_or_export (
 			MSG_DATA_PTR(message),
 			&found_tags,
 			&written_bytes_total,
-			MSG_DATA_LENGTH(message),
+			(rrr_length) MSG_DATA_LENGTH(message),
 			definition,
 			NULL, // Process all elements
-			__rrr_array_collection_pack_callback
+			__rrr_array_array_pack_callback
 	)) != 0) {
-		RRR_MSG_0("Error while converting array in rrr_array_new_message_from_collection return was %i\n", ret);
+		RRR_MSG_0("Error while converting array in rrr_array_new_message_from_array return was %i\n", ret);
 		goto out;
 	}
 
-	if (written_bytes_total != (ssize_t) total_data_length) {
-		RRR_BUG("Length mismatch after assembling message in rrr_array_new_message %li<>%lu\n",
-				written_bytes_total, MSG_DATA_LENGTH(message));
+	if (written_bytes_total != total_data_length) {
+		RRR_BUG("Length mismatch after assembling message in rrr_array_new_message %" PRIrrrbl "<>%llu\n",
+				written_bytes_total, (long long unsigned) MSG_DATA_LENGTH(message));
 	}
 
 	*final_message = (struct rrr_msg_msg *) message;
@@ -877,8 +1122,8 @@ int rrr_array_message_iterate (
 		rrr_length elements = rrr_be32toh(data_packed->elements);
 
 		if (pos + tag_length + total_length > end) {
-			RRR_MSG_0("Length of type %u index %i in array message exceeds total length (%u > %li)\n",
-					type, i, total_length, end - pos);
+			RRR_MSG_0("Length of type %u index %i in array message exceeds total length (%u > %lli)\n",
+					type, i, total_length, (long long int) (end - pos));
 			ret = 1;
 			goto out;
 		}
@@ -930,15 +1175,15 @@ static int __rrr_array_message_iterate_values_callback (
 		void *arg
 ) {
 	struct rrr_array_message_iterate_values_callback_data *callback_data = arg;
-	return rrr_type_value_raw_with_tmp_do (
-		data_start,
-		type,
-		flags,
-		tag_length,
-		total_length,
-		element_count,
-		callback_data->callback,
-		callback_data->callback_arg
+	return rrr_type_value_with_tmp_do (
+			data_start,
+			type,
+			flags,
+			tag_length,
+			total_length,
+			element_count,
+			callback_data->callback,
+			callback_data->callback_arg
 	);
 }
 
@@ -1041,11 +1286,11 @@ int rrr_array_message_clone_value_by_tag (
 	);
 }
 
-struct rrr_array_message_append_to_collection_callback_data {
+struct rrr_array_message_append_to_array_callback_data {
 	struct rrr_array *target_tmp;
 };
 
-static int __rrr_array_message_append_to_collection_callback (
+static int __rrr_array_message_append_to_array_callback (
 		const char *data_start,
 		const struct rrr_type_definition *type,
 		rrr_type_flags flags,
@@ -1054,42 +1299,32 @@ static int __rrr_array_message_append_to_collection_callback (
 		rrr_length element_count,
 		void *arg
 ) {
-	struct rrr_array_message_append_to_collection_callback_data *callback_data = arg;
+	struct rrr_array_message_append_to_array_callback_data *callback_data = arg;
+
 	int ret = 0;
 
 	struct rrr_type_value *template = NULL;
-	if ((ret = rrr_type_value_new (
+
+	if ((ret = rrr_type_value_new_and_unpack (
 			&template,
 			type,
+			data_start,
 			flags,
 			tag_length,
-			data_start,
 			total_length,
-			NULL,
-			element_count,
-			NULL,
-			total_length
+			element_count
 	)) != 0) {
-		RRR_MSG_0("Could not allocate value in __rrr_array_message_append_to_collection_callbackn\n");
+		RRR_MSG_0("Failed to unpack value of type '%s' index %i of array message\n", type->identifier, RRR_LL_COUNT(callback_data->target_tmp));
 		goto out;
 	}
 
-	// Append immediately to array to manage memory
 	RRR_LL_APPEND(callback_data->target_tmp, template);
 
-	memcpy (template->data, data_start + tag_length, total_length);
-
-	if (template->definition->unpack(template) != 0) {
-		RRR_MSG_0("Error while converting endianess for type '%s' index %i of array message\n", type->identifier, RRR_LL_COUNT(callback_data->target_tmp));
-		ret = 1;
-		goto out;
-	}
-
 	out:
 	return ret;
 }
 
-int rrr_array_message_append_to_collection (
+int rrr_array_message_append_to_array (
 		uint16_t *array_version,
 		struct rrr_array *target,
 		const struct rrr_msg_msg *message_orig
@@ -1098,13 +1333,13 @@ int rrr_array_message_append_to_collection (
 
 	struct rrr_array target_tmp = {0};
 
-	struct rrr_array_message_append_to_collection_callback_data callback_data = {
+	struct rrr_array_message_append_to_array_callback_data callback_data = {
 			&target_tmp
 	};
 
 	if ((ret =  rrr_array_message_iterate (
 			message_orig,
-			__rrr_array_message_append_to_collection_callback,
+			__rrr_array_message_append_to_array_callback,
 			&callback_data
 	)) != 0) {
 		goto out;
