@@ -2,7 +2,7 @@
 
 Read Route Record
 
-Copyright (C) 2019-2020 Atle Solbakken atle@goliathdns.no
+Copyright (C) 2019-2021 Atle Solbakken atle@goliathdns.no
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,12 +27,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../rrr_types.h"
 #include "../util/linked_list.h"
 
+struct rrr_nullsafe_str;
+struct rrr_type_value;
+
 struct rrr_http_field {
 	RRR_LL_NODE(struct rrr_http_field);
-	char *name;
-	char *value;
-	char *content_type;
-	rrr_length value_size;
+	struct rrr_nullsafe_str *name;
+	struct rrr_nullsafe_str *content_type;
+	struct rrr_nullsafe_str *value;
+	struct rrr_type_value *value_orig;
 };
 
 struct rrr_http_field_collection {
@@ -42,19 +45,23 @@ struct rrr_http_field_collection {
 void rrr_http_field_destroy (
 		struct rrr_http_field *field
 );
-int rrr_http_field_new_no_value (
+int rrr_http_field_new_no_value_raw (
 		struct rrr_http_field **target,
 		const char *name,
 		rrr_length name_length
 );
-int rrr_http_field_set_content_type (
-		struct rrr_http_field *target,
-		const char *content_type
+int rrr_http_field_new_no_value (
+		struct rrr_http_field **target,
+		const struct rrr_nullsafe_str *nullsafe
 );
-int rrr_http_field_set_value (
+int rrr_http_field_content_type_set (
+		struct rrr_http_field *target,
+		const struct rrr_nullsafe_str *content_type
+);
+int rrr_http_field_value_set (
 		struct rrr_http_field *target,
 		const char *value,
-		rrr_length value_size
+		rrr_length value_length
 );
 int rrr_http_field_collection_iterate_const (
 		const struct rrr_http_field_collection *fields,
@@ -70,24 +77,31 @@ void rrr_http_field_collection_clear (
 int rrr_http_field_collection_add (
 		struct rrr_http_field_collection *fields,
 		const char *name,
+		rrr_length name_length,
 		const char *value,
-		rrr_length value_size,
-		const char *content_type
+		rrr_length value_length,
+		const char *content_type,
+		rrr_length content_type_length,
+		const struct rrr_type_value *value_orig
 );
-rrr_length rrr_http_field_collection_get_total_length (
+int rrr_http_field_collection_iterate_as_strings (
+		const struct rrr_http_field_collection *fields,
+		int (*callback)(const struct rrr_nullsafe_str *name, const struct rrr_nullsafe_str *value, const struct rrr_nullsafe_str *content_type, void *arg),
+		void *callback_arg
+);
+int rrr_http_field_collection_to_urlencoded_form_data (
+		struct rrr_nullsafe_str **target,
 		struct rrr_http_field_collection *fields
 );
-const struct rrr_http_field *rrr_http_field_collection_get_field (
-		struct rrr_http_field_collection *fields,
-		const char *name
-);
-char *rrr_http_field_collection_to_urlencoded_form_data (
-		rrr_length *output_size,
+int rrr_http_field_collection_to_raw_form_data (
+		struct rrr_nullsafe_str **target,
 		struct rrr_http_field_collection *fields
 );
-char *rrr_http_field_collection_to_raw_form_data (
-		rrr_length *output_size,
-		struct rrr_http_field_collection *fields
+#ifdef RRR_WITH_JSONC
+int rrr_http_field_collection_to_json (
+		struct rrr_nullsafe_str **target,
+		const struct rrr_http_field_collection *fields
 );
+#endif
 
 #endif /* RRR_HTTP_FIELDS_H */
