@@ -27,14 +27,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../log.h"
 #include "../allocator.h"
 #include "http_part.h"
+#include "http_part_multipart.h"
 #include "http_fields.h"
 #include "http_util.h"
 #include "http_header_fields.h"
-#include "../string_builder.h"
+#include "../array.h"
+#include "../helpers/string_builder.h"
 #include "../helpers/nullsafe_str.h"
 #include "../util/macro_utils.h"
 #include "../util/base64.h"
 #include "../util/gnu.h"
+#include "../util/posix.h"
 
 int __rrr_http_part_content_type_equals (
 		struct rrr_http_part *part,
@@ -77,6 +80,7 @@ const struct rrr_http_field *__rrr_http_part_header_field_subvalue_get (
 void rrr_http_part_destroy (struct rrr_http_part *part) {
 	RRR_LL_DESTROY(part, struct rrr_http_part, rrr_http_part_destroy(node));
 	rrr_http_header_field_collection_clear(&part->headers);
+	rrr_array_collection_clear(&part->arrays);
 	RRR_LL_DESTROY(&part->chunks, struct rrr_http_chunk, rrr_free(node));
 	rrr_http_field_collection_clear(&part->fields);
 	rrr_nullsafe_str_destroy_if_not_null(&part->request_uri_nullsafe);
@@ -95,7 +99,7 @@ int rrr_http_part_new (struct rrr_http_part **result) {
 
 	struct rrr_http_part *part = rrr_allocate (sizeof(*part));
 	if (part == NULL) {
-		RRR_MSG_0("Could not allocate memory in rrr_http_part_new\n");
+		RRR_MSG_0("Could not allocate memory in %s\n", __func__);
 		ret = 1;
 		goto out;
 	}
@@ -116,7 +120,7 @@ int rrr_http_part_prepare (struct rrr_http_part **part) {
 		*part = NULL;
 	}
 	if ((ret = rrr_http_part_new(part)) != 0) {
-		RRR_MSG_0("Could not create HTTP part in rrr_http_part_prepare\n");
+		RRR_MSG_0("Could not create HTTP part in %s\n", __func__);
 		goto out;
 	}
 
@@ -128,57 +132,43 @@ const struct rrr_http_header_field *rrr_http_part_header_field_get (
 		const struct rrr_http_part *part,
 		const char *name
 ) {
-	RRR_LL_ITERATE_BEGIN(&part->headers, struct rrr_http_header_field);
-		if (rrr_nullsafe_str_cmpto_case(node->name, name) == 0) {
-			if (node->definition == NULL || node->definition->parse == NULL) {
-				RRR_BUG("Attempted to retrieve field %s which was not parsed in rrr_http_part_header_field_get, definition must be added\n",
-						name);
-			}
-			return node;
-		}
-	RRR_LL_ITERATE_END();
-	return NULL;
+	return rrr_http_header_field_collection_get(&part->headers, name);
 }
 
 const struct rrr_http_header_field *rrr_http_part_header_field_get_raw (
 		const struct rrr_http_part *part,
 		const char *name
 ) {
-	RRR_LL_ITERATE_BEGIN(&part->headers, struct rrr_http_header_field);
-		if (rrr_nullsafe_str_cmpto_case(node->name, name) == 0) {
-			return node;
-		}
-	RRR_LL_ITERATE_END();
-	return NULL;
+	return rrr_http_header_field_collection_get_raw(&part->headers, name);
 }
 
 const struct rrr_http_header_field *rrr_http_part_header_field_get_with_value_case (
 		const struct rrr_http_part *part,
 		const char *name_lowercase,
 		const char *value_anycase
+) {
+	return rrr_http_header_field_collection_get_with_value_case(&part->headers, name_lowercase, value_anycase);
+}
+
+struct rrr_http_header_field *__rrr_http_part_header_field_get (
+		struct rrr_http_part *part,
+		const char *name
 ) {
 	RRR_LL_ITERATE_BEGIN(&part->headers, struct rrr_http_header_field);
-		if (rrr_nullsafe_str_cmpto(node->name, name_lowercase) == 0) {
-			if (node->definition == NULL || node->definition->parse == NULL) {
-				RRR_HTTP_UTIL_SET_TMP_NAME_FROM_NULLSAFE(name,node->name);
-				RRR_BUG("BUG: Attempted to retrieve field %s which was not parsed in rrr_http_part_header_field_get_with_value_case, definition must be added\n",
-						name);
-			}
-			if (rrr_nullsafe_str_cmpto_case(node->value, value_anycase) == 0) {
-				return node;
-			}
-		}
+		if (rrr_nullsafe_str_cmpto(node->name, name) == 0)
+			return node;
 	RRR_LL_ITERATE_END();
+
 	return NULL;
 }
 
 struct rrr_http_chunk *rrr_http_part_chunk_new (
-		rrr_length chunk_start,
-		rrr_length chunk_length
+		rrr_biglength chunk_start,
+		rrr_biglength chunk_length
 ) {
 	struct rrr_http_chunk *new_chunk = rrr_allocate(sizeof(*new_chunk));
 	if (new_chunk == NULL) {
-		RRR_MSG_0("Could not allocate memory for chunk in rrr_http_part_chunk_new\n");
+		RRR_MSG_0("Could not allocate memory for chunk in %s\n", __func__);
 		return NULL;
 	}
 
@@ -220,6 +210,25 @@ int rrr_http_part_header_field_push (
 	return ret;
 }
 
+int rrr_http_part_header_field_push_nullsafe (
+		struct rrr_http_part *part,
+		const char *name,
+		const struct rrr_nullsafe_str *value
+) {
+	int ret = 0;
+
+	struct rrr_http_header_field *field = NULL;
+
+	if ((ret = rrr_http_header_field_new_with_value_nullsafe(&field, name, value)) != 0) {
+		goto out;
+	}
+
+	RRR_LL_APPEND(&part->headers, field);
+
+	out:
+	return ret;
+}
+
 int rrr_http_part_header_field_push_if_not_exists (
 		struct rrr_http_part *part,
 		const char *name,
@@ -241,6 +250,37 @@ int rrr_http_part_header_field_push_and_replace (
 	return rrr_http_part_header_field_push(part, name, value);
 }
 
+int rrr_http_part_header_field_push_subvalue (
+		struct rrr_http_part *part,
+		const char *field,
+		const char *name,
+		const char *value
+) {
+	int ret = 0;
+
+	struct rrr_http_header_field *content_type = __rrr_http_part_header_field_get (part, "content-type");
+	if (content_type == NULL) {
+		RRR_BUG("BUG: Field %s was not present in %s\n", field, __func__);
+	}
+
+	if ((ret = rrr_http_field_collection_add(
+			&content_type->fields,
+			name,
+			rrr_length_from_size_t_bug_const(strlen(name)),
+			value,
+			rrr_length_from_size_t_bug_const(strlen(value)),
+			NULL,
+			0,
+			NULL
+	)) != 0) {
+		RRR_MSG_0("Failed to add value to field collection in %s\n", __func__);
+		goto out;
+	}
+
+	out:
+	return ret;
+}
+
 int rrr_http_part_fields_iterate_const (
 		const struct rrr_http_part *part,
 		int (*callback)(const struct rrr_http_field *field, void *callback_arg),
@@ -280,7 +320,14 @@ int rrr_http_part_chunks_iterate (
 	}
 
 	if (RRR_LL_COUNT(&part->chunks) == 0) {
-		ret = callback(0, 1, data_start, data_end - data_start, part->data_length, callback_arg);
+		ret = callback (
+				0,
+				1,
+				data_start,
+				rrr_biglength_from_ptr_sub_bug_const(data_end, data_start),
+				part->data_length,
+				callback_arg
+		);
 		goto out;
 	}
 
@@ -291,7 +338,7 @@ int rrr_http_part_chunks_iterate (
 		const char *data_start = data_ptr + node->start;
 
 		if (data_start + node->length > data_end) {
-			RRR_BUG("Chunk end overrun in rrr_http_part_chunks_iterate\n");
+			RRR_BUG("Chunk end overrun in %s\n", __func__);
 		}
 
 		// NOTE : Length might be 0
@@ -314,7 +361,7 @@ static int __rrr_http_part_query_string_parse (
 	int ret = 0;
 
 	char *buf = NULL;
-	size_t buf_pos = 0;
+	rrr_length buf_pos = 0;
 	struct rrr_http_field *field_tmp = NULL;
 	struct rrr_http_field *value_target = NULL;
 
@@ -326,8 +373,16 @@ static int __rrr_http_part_query_string_parse (
 		start++;
 	}
 
-	if ((buf = rrr_allocate((end - start) + 1)) == NULL) {
-		RRR_MSG_0("Could not allocate memory for buffer in __rrr_http_part_query_string_parse\n");
+	rrr_length to_allocate = 0;
+	// +1 to accomodate zero termination
+	if (rrr_length_from_ptr_sub_err(&to_allocate, end + 1, start) != 0) {
+		RRR_MSG_0("Data too long while parsing HTTP query string\n");
+		ret = RRR_HTTP_PARSE_SOFT_ERR;
+		goto out;
+	}
+
+	if ((buf = rrr_allocate(to_allocate)) == NULL) {
+		RRR_MSG_0("Could not allocate memory for buffer in %s\n", __func__);
 		ret = RRR_HTTP_PARSE_HARD_ERR;
 		goto out;
 	}
@@ -336,7 +391,7 @@ static int __rrr_http_part_query_string_parse (
 		int push_no_value = 0;
 		int end_is_near = 0;
 
-		unsigned char c = *start;
+		unsigned char c = (unsigned char) *start;
 
 		if (start + 1 >= end || c == ' ' || c == '\t' || c == '\r' || c == '\n') {
 			end_is_near = 1;
@@ -363,10 +418,10 @@ static int __rrr_http_part_query_string_parse (
 			}
 
 			if (result > 0xff) {
-				RRR_BUG("Result after converting %%-sequence too big in __rrr_http_part_query_string_parse\n");
+				RRR_BUG("Result after converting %%-sequence too big in %s\n", __func__);
 			}
 
-			c = result;
+			c = (unsigned char) result;
 			start += 2; // One more ++ at the end of the loop
 
 			if (start + 1 >= end || *(start+1) == ' ' || *(start+1) == '\t' || *(start+1) == '\r' || *(start+1) == '\n') {
@@ -392,26 +447,21 @@ static int __rrr_http_part_query_string_parse (
 			}
 		}
 
-		if (end_is_near) {
-			buf[buf_pos++] = c;
-			buf[buf_pos] = '\0';
+		buf[rrr_length_inc_bug_old_value(&buf_pos)] = (char) c;
+		buf[buf_pos] = '\0';
 
+		if (end_is_near) {
 			if (value_target == NULL) {
 				goto push_new_field_no_value;
 			}
-			else {
-				goto store_value;
-			}
+			goto store_value;
 		}
 
-		buf[buf_pos++] = c;
-		buf[buf_pos] = '\0';
-
 		goto increment;
 
 		store_value:
 			if (rrr_http_field_value_set(value_target, buf, buf_pos) != 0) {
-				RRR_MSG_0("Could not set value in __rrr_http_part_query_string_parse\n");
+				RRR_MSG_0("Could not set value in %s\n", __func__);
 				ret = RRR_HTTP_PARSE_HARD_ERR;
 				goto out;
 			}
@@ -424,7 +474,7 @@ static int __rrr_http_part_query_string_parse (
 		push_new_field:
 			if (buf_pos > 0) {
 				if (rrr_http_field_new_no_value_raw(&field_tmp, buf, buf_pos) != 0) {
-					RRR_MSG_0("Could not allocate new field in __rrr_http_part_query_string_parse\n");
+					RRR_MSG_0("Could not allocate new field in %s\n", __func__);
 					ret = RRR_HTTP_PARSE_HARD_ERR;
 					goto out;
 				}
@@ -458,7 +508,7 @@ static int __rrr_http_part_query_string_parse (
 
 static int __rrr_http_part_query_fields_from_uri_extract_callback (
 		const void *start,
-		size_t len_remaining,
+		rrr_nullsafe_len len_remaining,
 		void *arg
 ) {
 	const char *query_start = start;
@@ -475,7 +525,7 @@ static int __rrr_http_part_query_fields_from_uri_extract_callback (
 	}
 
 	if ((ret = __rrr_http_part_query_string_parse (&target->fields, query_start, query_end)) != 0) {
-		RRR_MSG_0("Error while parsing query string in __rrr_http_part_query_fields_from_uri_extract\n");
+		RRR_MSG_0("Error while parsing query string in %s\n", __func__);
 		goto out;
 	}
 
@@ -483,13 +533,38 @@ static int __rrr_http_part_query_fields_from_uri_extract_callback (
 	return ret;
 }
 
-int rrr_http_part_fields_from_uri_extract (
+static int __rrr_http_part_fields_from_uri_extract (
 		struct rrr_http_part *target
 ) {
 	return rrr_nullsafe_str_chr(target->request_uri_nullsafe, '?', __rrr_http_part_query_fields_from_uri_extract_callback, target);
 }
 
-int rrr_http_part_fields_from_post_extract (
+struct rrr_http_part_fields_from_post_data_extract_json_callback_data {
+	struct rrr_http_part *target;
+};
+
+static int __rrr_http_part_fields_from_post_extract_json_callback (
+		const struct rrr_array *array,
+		void *arg
+) {
+	struct rrr_http_part_fields_from_post_data_extract_json_callback_data *callback_data = arg;
+
+	int ret = 0;
+
+	struct rrr_array *new_array = NULL;
+
+	if ((ret = rrr_array_clone (&new_array, array)) != 0) {
+		RRR_MSG_0("Failed to clone array in %s\n", __func__);
+		goto out;
+	}
+
+	RRR_LL_APPEND(&callback_data->target->arrays, new_array);
+
+	out:
+	return ret;
+}
+
+static int __rrr_http_part_fields_from_post_extract (
 		struct rrr_http_part *target,
 		const char *data_ptr
 ) {
@@ -499,9 +574,8 @@ int rrr_http_part_fields_from_post_extract (
 
 	if (__rrr_http_part_content_type_equals(target, "application/x-www-form-urlencoded")) {
 		RRR_HTTP_PART_DECLARE_DATA_START_AND_END(target, data_ptr);
-
 		if ((ret = __rrr_http_part_query_string_parse (&target->fields, data_start, data_end)) != 0) {
-			RRR_MSG_0("Error while parsing query string in rrr_http_part_post_and_query_fields_extract\n");
+			RRR_MSG_0("Error while parsing query string in %s\n", __func__);
 			goto out;
 		}
 	}
@@ -509,6 +583,13 @@ int rrr_http_part_fields_from_post_extract (
 		RRR_LL_ITERATE_BEGIN(target, struct rrr_http_part);
 			RRR_HTTP_PART_DECLARE_DATA_START_AND_END(node, data_ptr);
 
+			if (data_length > RRR_LENGTH_MAX) {
+				RRR_MSG_0("Multipart form value too big, cannot be stored (%" PRIrrrbl ">%llu)\n",
+					data_length, (unsigned long long) RRR_LENGTH_MAX);
+				ret = RRR_HTTP_PARSE_SOFT_ERR;
+				goto out;
+			}
+
 			const struct rrr_http_field *field_name = __rrr_http_part_header_field_subvalue_get(node, "content-disposition", "name");
 			if (field_name == NULL || !rrr_nullsafe_str_isset(field_name->value)) {
 				RRR_DBG_1("Warning: Unknown field or invalid content-disposition of multipart part\n");
@@ -516,13 +597,17 @@ int rrr_http_part_fields_from_post_extract (
 			}
 
 			if ((ret = rrr_http_field_new_no_value(&field_tmp, field_name->value)) != 0) {
-				RRR_MSG_0("Could not create new field in rrr_http_part_post_and_query_fields_extract\n");
+				RRR_MSG_0("Could not create new field in %s\n", __func__);
 				goto out;
 			}
 
-			if (data_end - data_start > 0) {
-				if ((ret = rrr_http_field_value_set(field_tmp, data_start, data_end - data_start)) != 0) {
-					RRR_MSG_0("Could not set value of field in rrr_http_part_post_and_query_fields_extract\n");
+			if (data_length > 0) {
+				if ((ret = rrr_http_field_value_set (
+						field_tmp,
+						data_start,
+						rrr_length_from_biglength_bug_const(data_length)
+				)) != 0) {
+					RRR_MSG_0("Could not set value of field in %s\n", __func__);
 					goto out;
 				}
 			}
@@ -533,7 +618,7 @@ int rrr_http_part_fields_from_post_extract (
 						field_tmp,
 						field_content_type->value
 				)) != 0) {
-					RRR_MSG_0("Could not set content type of field in rrr_http_part_post_and_query_fields_extract\n");
+					RRR_MSG_0("Could not set content type of field in %s\n", __func__);
 					goto out;
 				}
 			}
@@ -542,6 +627,35 @@ int rrr_http_part_fields_from_post_extract (
 			field_tmp = NULL;
 		RRR_LL_ITERATE_END();
 	}
+#ifdef RRR_WITH_JSONC
+	else if (__rrr_http_part_content_type_equals(target, "application/json")) {
+		RRR_HTTP_PART_DECLARE_DATA_START_AND_END(target, data_ptr);
+
+		rrr_length json_length;
+		if (rrr_length_from_biglength_err(&json_length, data_length) != 0) {
+			RRR_MSG_0("JSON data in HTTP body too big, cannot be stored (%" PRIrrrbl ">%llu)\n",
+					data_length, (unsigned long long) RRR_LENGTH_MAX);
+			ret = RRR_HTTP_PARSE_SOFT_ERR;
+			goto out;
+		}
+
+		struct rrr_http_part_fields_from_post_data_extract_json_callback_data callback_data = {
+			target
+		};
+
+		if ((ret = rrr_http_util_json_to_arrays (
+				data_start,
+				json_length,
+				__rrr_http_part_fields_from_post_extract_json_callback,
+				&callback_data
+		)) != 0) {
+			RRR_MSG_0("Failed to parse JSON in HTTP request body, return was %i\n", ret);
+			// Mask hard errors (allocation failures)
+			ret = RRR_HTTP_PARSE_SOFT_ERR;
+			goto out;
+		}
+	}
+#endif
 
 	out:
 	if (field_tmp != NULL) {
@@ -550,6 +664,31 @@ int rrr_http_part_fields_from_post_extract (
 	return ret;
 }
 
+int rrr_http_part_multipart_and_fields_process (
+		struct rrr_http_part *part,
+		const char *data_or_null,
+		short no_body_parse
+) {
+	int ret = 0;
+
+	if ((ret = __rrr_http_part_fields_from_uri_extract(part)) != 0) {
+		goto out;
+	}
+
+	if (!no_body_parse && data_or_null != NULL) {
+		if ((ret = rrr_http_part_multipart_process(part, data_or_null)) != 0) {
+			goto out;
+		}
+
+		if ((ret = __rrr_http_part_fields_from_post_extract(part, data_or_null)) != 0) {
+			goto out;
+		}
+	}
+
+	out:
+	return ret;
+}
+
 int rrr_http_part_chunks_merge (
 		char **result_data,
 		struct rrr_http_part *part,
@@ -557,36 +696,47 @@ int rrr_http_part_chunks_merge (
 ) {
 	int ret = RRR_HTTP_OK;
 
+	char *data_new = NULL;
+
 	if (part->is_chunked == 0) {
 		goto out;
 	}
 
 	*result_data = NULL;
 
-	char *data_new = NULL;
-	const size_t top_length = RRR_HTTP_PART_TOP_LENGTH(part);
-	size_t new_buf_size = 0;
-
-	new_buf_size += top_length;
+	const rrr_biglength top_length = RRR_HTTP_PART_TOP_LENGTH(part);
+	rrr_biglength new_buf_size = top_length;
 
 	RRR_LL_ITERATE_BEGIN(&part->chunks, struct rrr_http_chunk);
 		new_buf_size += node->length;
+		if (new_buf_size < node->length) {
+			RRR_MSG_0("Overflow while merging HTTP chunks\n");
+			ret = 1;
+			goto out;
+		}
 	RRR_LL_ITERATE_END();
 
 	if ((data_new = rrr_allocate(new_buf_size)) == NULL) {
-		RRR_MSG_0("Could not allocate memory in rrr_http_part_chunks_merge\n");
+		RRR_MSG_0("Could not allocate memory in %s\n", __func__);
 		ret = RRR_HTTP_HARD_ERROR;
 		goto out;
 	}
 
-	size_t wpos = 0;
+	rrr_biglength wpos = 0;
 
-	memcpy(data_new, data_ptr, top_length);
+	RRR_SIZE_CHECK(top_length,"Merge HTTP chunks",ret = 1; goto out);
+
+	rrr_memcpy(data_new, data_ptr, top_length);
 	wpos += top_length;
 
 	RRR_LL_ITERATE_BEGIN(&part->chunks, struct rrr_http_chunk);
-		memcpy(data_new + wpos, data_ptr + node->start, node->length);
+		RRR_SIZE_CHECK(node->length,"Merge HTTP chunk node",ret = 1; goto out);
+		rrr_memcpy(data_new + wpos, data_ptr + node->start, node->length);
 		wpos += node->length;
+		if (wpos < node->length) {
+			// Should discover this during allocation loop
+			RRR_BUG("Overflow while merging HTTP chunk\n");
+		}
 		RRR_LL_ITERATE_SET_DESTROY();
 	RRR_LL_ITERATE_END_CHECK_DESTROY(&part->chunks, 0; rrr_free(node));
 
@@ -594,10 +744,10 @@ int rrr_http_part_chunks_merge (
 	part->data_length = wpos;
 
 	*result_data = data_new;
+	data_new = NULL;
 
-	// goto out; out_free:
-	// RRR_FREE_IF_NOT_NULL(data_new); -- Enable if needed
 	out:
+	RRR_FREE_IF_NOT_NULL(data_new);
 	return ret;
 }
 
