@@ -20,7 +20,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <unistd.h>
-
 #include <stdlib.h>
 
 #include "log.h"
@@ -232,56 +231,15 @@ int rrr_instance_load_and_save (
 	return 0;
 }
 
-struct add_instance_data {
-	struct rrr_instance_collection *instances;
-	struct rrr_instance_friend_collection *collection;
-};
-
-static int __rrr_add_instance_callback(const char *value, void *_data) {
-	struct add_instance_data *data = _data;
-
-	int ret = 0;
-
-	struct rrr_instance *instance = rrr_instance_find(data->instances, value);
-
-	if (instance == NULL) {
-		RRR_MSG_0("Could not find instance '%s'\n", value);
-		ret = 1;
-		goto out;
-	}
-
-	RRR_DBG_1("Added %s\n", INSTANCE_M_NAME(instance));
-
-	rrr_instance_friend_collection_append(data->collection, instance);
-
-	out:
-	return ret;
-}
-
 static int __rrr_instance_parse_topic_filter (
 		struct rrr_instance *data
 ) {
-	int ret = 0;
-
-	struct rrr_instance_config_data *config = data->config;
-
-	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_UTF8_DEFAULT_NULL("topic_filter", topic_filter);
-
-	if (data->topic_filter != NULL) {
-		if (rrr_mqtt_topic_filter_validate_name(data->topic_filter) != 0) {
-			RRR_MSG_0("Invalid topic_filter setting found for instance %s\n", config->name);
-			ret = 1;
-			goto out;
-		}
-		if (rrr_mqtt_topic_tokenize(&data->topic_first_token, data->topic_filter) != 0) {
-			RRR_MSG_0("Error while tokenizing topic filter in __rrr_instance_parse_topic_filter\n");
-			ret = 1;
-			goto out;
-		}
-	}
-
-	out:
-	return ret;
+	return rrr_instance_config_parse_optional_topic_filter (
+			&data->topic_first_token,
+			&data->topic_filter,
+			data->config,
+			"topic_filter"
+	);
 }
 
 static int __rrr_instance_parse_misc (
@@ -295,6 +253,7 @@ static int __rrr_instance_parse_misc (
 		int do_enable_buffer;
 		int do_enable_backstop;
 		int do_duplicate;
+		int do_topic_filter_invert;
 	} data_tmp;
 
 	struct data *data = &data_tmp;
@@ -316,6 +275,10 @@ static int __rrr_instance_parse_misc (
 	if (data->do_duplicate) {
 		data_final->misc_flags |= RRR_INSTANCE_MISC_OPTIONS_DUPLICATE;
 	}
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("topic_filter_invert", do_topic_filter_invert, 0);
+	if (data->do_topic_filter_invert) {
+		data_final->misc_flags |= RRR_INSTANCE_MISC_OPTIONS_TOPIC_FILTER_INVERT;
+	}
 
 	out:
 	return ret;
@@ -332,18 +295,14 @@ static int __rrr_instance_add_wait_for_instances (
 			instance->module_data->module_name
 	);
 
-	struct add_instance_data add_data = {0};
-
-	add_data.collection = &instance->wait_for;
-	add_data.instances = instances;
-
-	struct rrr_instance_config_data *instance_config = instance->config;
-
-	if ((ret = rrr_settings_traverse_split_commas_silent_fail (
-			instance_config->settings, "wait_for",
-			&__rrr_add_instance_callback, &add_data
-	))!= 0) {
-		RRR_MSG_0("Error while adding wait for for instance %s\n", instance->module_data->instance_name);
+	if ((ret = rrr_instance_config_friend_collection_populate_from_config (
+			&instance->wait_for,
+			instances,
+			instance->config,
+			"wait_for"
+	)) != 0) {
+		RRR_MSG_0("Error while adding wait for for instance %s\n",
+			INSTANCE_M_NAME(instance));
 		goto out;
 	}
 
@@ -362,18 +321,14 @@ static int __rrr_instance_add_senders (
 			INSTANCE_M_MODULE_NAME(instance)
 	);
 
-	struct rrr_instance_config_data *instance_config = instance->config;
-
-	struct add_instance_data add_data = {0};
-	add_data.instances = instances;
-	add_data.collection = &instance->senders;
-
-	if ((ret = rrr_settings_traverse_split_commas_silent_fail (
-			instance_config->settings, "senders",
-			&__rrr_add_instance_callback, &add_data
-	))!= 0) {
+	if ((ret = rrr_instance_config_friend_collection_populate_from_config (
+			&instance->senders,
+			instances,
+			instance->config,
+			"senders"
+	)) != 0) {
 		RRR_MSG_0("Error while adding senders for instance %s\n",
-				INSTANCE_M_NAME(instance)
+			INSTANCE_M_NAME(instance)
 		);
 		goto out;
 	}
@@ -450,13 +405,23 @@ static int __rrr_instance_add_senders (
 	return ret;
 }
 
+int rrr_instance_has_sender (
+		const struct rrr_instance *instance,
+		const struct rrr_instance *sender
+) {
+	return rrr_instance_friend_collection_check_exists (
+			&instance->senders,
+			sender
+	);
+}
+
 void rrr_instance_collection_clear (
 		struct rrr_instance_collection *target
 ) {
 	RRR_LL_DESTROY(target, struct rrr_instance, __rrr_instance_destroy(node));
 }
 
-unsigned int rrr_instance_collection_count (
+int rrr_instance_collection_count (
 		struct rrr_instance_collection *collection
 ) {
 	if (RRR_LL_COUNT(collection) < 0) {
@@ -672,7 +637,45 @@ static void __rrr_instance_thread_stats_instance_cleanup (
 	pthread_mutex_unlock(&thread->mutex);
 }
 
-#include "fork.h"
+struct rrr_instance_count_receivers_of_self_callback_data {
+	struct rrr_instance *self;
+	rrr_length count;
+};
+
+static int __rrr_instance_count_receivers_of_self_callback (
+		struct rrr_instance *instance,
+		void *arg
+) {
+	struct rrr_instance_count_receivers_of_self_callback_data *callback_data = arg;
+	if (instance == callback_data->self) {
+		rrr_length_inc_bug(&callback_data->count);
+	}
+	return 0;
+}
+
+static rrr_length __rrr_instance_count_receivers_of_self (
+		struct rrr_instance *self
+) {
+	struct rrr_instance_collection *instances = self->module_data->all_instances;
+
+	struct rrr_instance_count_receivers_of_self_callback_data callback_data = {
+			self,
+			0
+	};
+
+	RRR_LL_ITERATE_BEGIN(instances, struct rrr_instance);
+		struct rrr_instance *instance = node;
+		if (instance != self) {
+			rrr_instance_friend_collection_iterate (
+					&instance->senders,
+					__rrr_instance_count_receivers_of_self_callback,
+					&callback_data
+			);
+		}
+	RRR_LL_ITERATE_END();
+
+	return callback_data.count;
+}
 
 static void *__rrr_instance_thread_entry_intermediate (
 		struct rrr_thread *thread
@@ -740,9 +743,9 @@ static int __rrr_instance_thread_preload_enable_duplication_as_needed (
 	int ret = 0;
 
 	if (INSTANCE_D_FLAGS(thread_data) & RRR_INSTANCE_MISC_OPTIONS_DUPLICATE) {
-		int slots = rrr_instance_count_receivers_of_self(INSTANCE_D_INSTANCE(thread_data));
+		rrr_length slots = __rrr_instance_count_receivers_of_self(INSTANCE_D_INSTANCE(thread_data));
 
-		RRR_DBG_1("%s instance %s setting up duplicated output buffer, %i readers detected\n",
+		RRR_DBG_1("%s instance %s setting up duplicated output buffer, %" PRIrrrl " readers detected\n",
 				INSTANCE_D_MODULE_NAME(thread_data), INSTANCE_D_NAME(thread_data), slots);
 
 		if ((ret = rrr_message_broker_setup_split_output_buffer (
@@ -837,7 +840,7 @@ static int __rrr_instance_collection_start_threads_check_wait_for_callback (
 int rrr_instances_create_and_start_threads (
 		struct rrr_thread_collection **thread_collection_target,
 		struct rrr_instance_collection *instances,
-		struct rrr_config *global_config,
+		struct rrr_instance_config_collection *config,
 		struct cmd_data *cmd,
 		struct rrr_stats_engine *stats,
 		struct rrr_message_broker *message_broker,
@@ -874,7 +877,7 @@ int rrr_instances_create_and_start_threads (
 		init_data.module = instance->module_data;
 		init_data.senders = &instance->senders;
 		init_data.cmd_data = cmd;
-		init_data.global_config = global_config;
+		init_data.global_config = config;
 		init_data.instance_config = instance->config;
 		init_data.stats = stats;
 		init_data.message_broker = message_broker;
@@ -949,23 +952,24 @@ int rrr_instances_create_and_start_threads (
 		return ret;
 }
 
-int rrr_instance_create_from_config (
+int rrr_instances_create_from_config (
 		struct rrr_instance_collection *instances,
-		struct rrr_config *config,
+		struct rrr_instance_config_collection *config,
 		const char **library_paths
 ) {
 	int ret = 0;
-	for (int i = 0; i < config->module_count; i++) {
-		ret = rrr_instance_load_and_save(instances, config->configs[i], library_paths);
+
+	RRR_LL_ITERATE_BEGIN(config, struct rrr_instance_config_data);
+		ret = rrr_instance_load_and_save(instances, node, library_paths);
 		if (ret != 0) {
 			RRR_MSG_0("Loading of instance failed for %s. Library paths used:\n",
-					config->configs[i]->name);
+					node->name);
 			for (int j = 0; *library_paths[j] != '\0'; j++) {
 				RRR_MSG_0("-> %s\n", library_paths[j]);
 			}
 			goto out;
 		}
-	}
+	RRR_LL_ITERATE_END();
 
 	RRR_LL_ITERATE_BEGIN(instances, struct rrr_instance);
 		struct rrr_instance *instance = node;
@@ -999,48 +1003,8 @@ int rrr_instance_create_from_config (
 	return ret;
 }
 
-struct rrr_instance_count_receivers_of_self_callback_data {
-	struct rrr_instance *self;
-	int count;
-};
-
-static int __rrr_instance_count_receivers_of_self_callback (
-		struct rrr_instance *instance,
-		void *arg
-) {
-	struct rrr_instance_count_receivers_of_self_callback_data *callback_data = arg;
-	if (instance == callback_data->self) {
-		callback_data->count++;
-	}
-	return 0;
-}
-
-int rrr_instance_count_receivers_of_self (
-		struct rrr_instance *self
-) {
-	struct rrr_instance_collection *instances = self->module_data->all_instances;
-
-	struct rrr_instance_count_receivers_of_self_callback_data callback_data = {
-			self,
-			0
-	};
-
-	RRR_LL_ITERATE_BEGIN(instances, struct rrr_instance);
-		struct rrr_instance *instance = node;
-		if (instance != self) {
-			rrr_instance_friend_collection_iterate (
-					&instance->senders,
-					__rrr_instance_count_receivers_of_self_callback,
-					&callback_data
-			);
-		}
-	RRR_LL_ITERATE_END();
-
-	return callback_data.count;
-}
-
 int rrr_instance_default_set_output_buffer_ratelimit_when_needed (
-		int *delivery_entry_count,
+		unsigned int *delivery_entry_count,
 		int *delivery_ratelimit_active,
 		struct rrr_instance_runtime_data *thread_data
 ) {
