@@ -23,16 +23,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define RRR_INSTANCES_H
 
 #include "modules.h"
-#include "configuration.h"
 #include "instance_friends.h"
 #include "threads.h"
 #include "poll_helper.h"
 #include "event/event.h"
 #include "util/linked_list.h"
 
-#define RRR_INSTANCE_MISC_OPTIONS_DISABLE_BUFFER   (1<<0)
-#define RRR_INSTANCE_MISC_OPTIONS_DISABLE_BACKSTOP (1<<1)
-#define RRR_INSTANCE_MISC_OPTIONS_DUPLICATE        (1<<2)
+#define RRR_INSTANCE_MISC_OPTIONS_DISABLE_BUFFER       (1<<0)
+#define RRR_INSTANCE_MISC_OPTIONS_DISABLE_BACKSTOP     (1<<1)
+#define RRR_INSTANCE_MISC_OPTIONS_DUPLICATE            (1<<2)
+#define RRR_INSTANCE_MISC_OPTIONS_TOPIC_FILTER_INVERT  (1<<3)
 
 struct rrr_stats_instance;
 struct rrr_cmodule;
@@ -40,6 +40,7 @@ struct rrr_fork_handler;
 struct rrr_stats_engine;
 struct rrr_message_broker;
 struct rrr_mqtt_topic_token;
+struct rrr_instance_config_collection;
 
 struct rrr_instance {
 	RRR_LL_NODE(struct rrr_instance);
@@ -99,7 +100,7 @@ struct rrr_instance_runtime_init_data {
 
 	// Shortcuts to other structures
 	struct rrr_instance_config_data *instance_config;
-	struct rrr_config *global_config;
+	struct rrr_instance_config_collection *global_config;
 	struct rrr_instance_module_data *module;
 	struct rrr_instance_friend_collection *senders;
 	struct rrr_stats_engine *stats;
@@ -148,6 +149,7 @@ struct rrr_instance_runtime_data {
 #define INSTANCE_D_TOPIC_STR(thread_data) thread_data->init_data.topic_str
 #define INSTANCE_D_CANCEL_CHECK_ARGS(thread_data) \
 		rrr_thread_signal_encourage_stop_check_and_update_watchdog_timer_void, INSTANCE_D_THREAD(thread_data)
+#define INSTANCE_D_INSTANCES(thread_data) thread_data->init_data.module->all_instances
 
 struct rrr_instance *rrr_instance_find_by_thread (
 		struct rrr_instance_collection *instances,
@@ -163,6 +165,10 @@ int rrr_instance_count_library_users (
 void rrr_instance_unload_all(
 		struct rrr_instance_collection *target
 );
+int rrr_instance_has_sender (
+		const struct rrr_instance *instance,
+		const struct rrr_instance *sender
+);
 void rrr_instance_collection_clear (
 		struct rrr_instance_collection *target
 );
@@ -178,7 +184,7 @@ struct rrr_instance *rrr_instance_find (
 		struct rrr_instance_collection *target,
 		const char *name
 );
-unsigned int rrr_instance_collection_count (
+int rrr_instance_collection_count (
 		struct rrr_instance_collection *collection
 );
 void rrr_instance_runtime_data_destroy_hard (
@@ -190,22 +196,19 @@ struct rrr_instance_runtime_data *rrr_instance_runtime_data_new (
 int rrr_instances_create_and_start_threads (
 		struct rrr_thread_collection **thread_collection_target,
 		struct rrr_instance_collection *instances,
-		struct rrr_config *global_config,
+		struct rrr_instance_config_collection *config,
 		struct cmd_data *cmd,
 		struct rrr_stats_engine *stats,
 		struct rrr_message_broker *message_broker,
 		struct rrr_fork_handler *fork_handler
 );
-int rrr_instance_create_from_config (
+int rrr_instances_create_from_config (
 		struct rrr_instance_collection *instances,
-		struct rrr_config *config,
+		struct rrr_instance_config_collection *config,
 		const char **library_paths
 );
-int rrr_instance_count_receivers_of_self (
-		struct rrr_instance *self
-);
 int rrr_instance_default_set_output_buffer_ratelimit_when_needed (
-		int *delivery_entry_count,
+		unsigned int *delivery_entry_count,
 		int *delivery_ratelimit_active,
 		struct rrr_instance_runtime_data *thread_data
 );
