@@ -143,6 +143,7 @@ void rrr_msg_holder_decref_while_locked_and_unlock (
 	else if (--(entry->usercount) == 0) {
 		RRR_FREE_IF_NOT_NULL(entry->message);
 		rrr_msg_holder_private_data_clear(entry);
+		rrr_instance_friend_collection_clear(&entry->nexthops);
 		entry->usercount = 1; // Avoid bug trap
 		rrr_msg_holder_unlock(entry);
 		__rrr_msg_holder_util_lock_destroy(entry);
@@ -177,6 +178,12 @@ void rrr_msg_holder_incref_while_locked (
 #endif
 }
 
+void rrr_msg_holder_incref_while_locked_void (
+		void *entry
+) {
+	rrr_msg_holder_incref_while_locked(entry);
+}
+
 void rrr_msg_holder_incref (
 		struct rrr_msg_holder *entry
 ) {
@@ -206,10 +213,10 @@ void rrr_msg_holder_decref_void (
 
 int rrr_msg_holder_new (
 		struct rrr_msg_holder **result,
-		ssize_t data_length,
+		rrr_biglength data_length,
 		const struct sockaddr *addr,
 		socklen_t addr_len,
-		int protocol,
+		uint8_t protocol,
 		void *message
 ) {
 	int ret = 0;
@@ -218,7 +225,7 @@ int rrr_msg_holder_new (
 
 	struct rrr_msg_holder *entry = rrr_allocate_group(sizeof(*entry), RRR_ALLOCATOR_GROUP_MSG_HOLDER);
 	if (entry == NULL) {
-		RRR_MSG_0("Could not allocate memory in message_holder_new\n");
+		RRR_MSG_0("Could not allocate memory in rrr_msg_holder_new\n");
 		ret = 1;
 		goto out;
 	}
@@ -231,18 +238,20 @@ int rrr_msg_holder_new (
 		goto out_free;
 	}
 
-	// Avoid usercount bug trap, write once again while holding the lock below
+	// Avoid usercount bug trap, initialize usercount once again later while holding the lock
 	entry->usercount = 99999;
 
 	rrr_msg_holder_lock(entry);
 
-	RRR_LL_NODE_INIT(entry);
+	// Ensure all fields are written to while lock is held
+	RRR_MESSAGE_HOLDER_ZERO_ALL(entry);
 
 	if (addr == NULL) {
 		memset(&entry->addr, '\0', sizeof(entry->addr));
 	}
 	else if (addr_len > sizeof(entry->addr)) {
-		RRR_BUG("Address too long (%u > %lu) in rrr_msg_holder_new\n", addr_len, sizeof(entry->addr));
+		RRR_BUG("Address too long (%llu > %llu) in rrr_msg_holder_new\n",
+			(unsigned long long) addr_len, (unsigned long long) sizeof(entry->addr));
 	}
 	else {
 		memcpy(&entry->addr, addr, addr_len);
@@ -277,29 +286,78 @@ int rrr_msg_holder_clone_no_data (
 		struct rrr_msg_holder **result,
 		const struct rrr_msg_holder *source
 ) {
-	int ret = rrr_msg_holder_new (
-			result,
+	int ret = 0;
+
+	*result = NULL;
+
+	struct rrr_msg_holder *entry = NULL;
+
+	if ((ret = rrr_msg_holder_new (
+			&entry,
 			0,
 			(struct sockaddr *) &source->addr,
 			source->addr_len,
 			source->protocol,
 			NULL
-	);
-
-	if (ret == 0) {
-		rrr_msg_holder_lock(*result);
-		(*result)->buffer_time = source->buffer_time;
-		(*result)->send_time = source->send_time;
-		rrr_msg_holder_unlock(*result);
+	)) != 0) {
+		goto out;
 	}
 
+	rrr_msg_holder_lock(entry);
+
+	entry->buffer_time = source->buffer_time;
+	entry->send_time = source->send_time;
+
+	ret = rrr_instance_friend_collection_append_from (&entry->nexthops, &source->nexthops);
+
+	rrr_msg_holder_unlock(entry);
+
+	if (ret != 0) {
+		RRR_MSG_0("Failed to clone nexthops list in %s\n", __func__);
+		goto out;
+	}
+
+	*result = entry;
+	entry = NULL;
+
+	out:
+	if (entry != NULL) {
+		rrr_msg_holder_decref(entry);
+	}
 	return ret;
 }
 
+void rrr_msg_holder_nexthops_reset (
+		struct rrr_msg_holder *entry
+) {
+	rrr_instance_friend_collection_clear (&entry->nexthops);
+}
+
+int rrr_msg_holder_nexthops_set (
+		struct rrr_msg_holder *entry,
+		const rrr_msg_holder_nexthops *hops
+) {
+	rrr_msg_holder_nexthops_reset(entry);
+
+	if (hops == NULL || RRR_LL_COUNT(hops) == 0) {
+		return 0;
+	}
+
+	return rrr_instance_friend_collection_append_from (&entry->nexthops, hops);
+}
+
+int rrr_msg_holder_nexthop_ok (
+		const struct rrr_msg_holder *entry,
+		const struct rrr_instance *instance
+) {
+	return rrr_instance_friend_collection_check_empty (&entry->nexthops) ||
+	       rrr_instance_friend_collection_check_exists (&entry->nexthops, instance);
+}
+
 void rrr_msg_holder_set_data_unlocked (
 		struct rrr_msg_holder *target,
 		void *message,
-		ssize_t message_data_length
+		rrr_biglength message_data_length
 ) {
 	RRR_FREE_IF_NOT_NULL(target->message);
 	target->message = message;
@@ -309,10 +367,10 @@ void rrr_msg_holder_set_data_unlocked (
 void rrr_msg_holder_set_unlocked (
 		struct rrr_msg_holder *target,
 		void *message,
-		ssize_t message_data_length,
+		rrr_biglength message_data_length,
 		const struct sockaddr *addr,
 		socklen_t addr_len,
-		int protocol
+		uint8_t protocol
 ) {
 	rrr_msg_holder_set_data_unlocked (target, message, message_data_length);
 	memcpy(&target->addr, addr, addr_len);
