@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "../log.h"
 #include "../allocator.h"
+#include "../util/posix.h"
 
 #include "message_holder.h"
 #include "message_holder_util.h"
@@ -33,10 +34,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 int rrr_msg_holder_util_new_with_empty_message (
 		struct rrr_msg_holder **result,
-		ssize_t message_data_length,
+		rrr_biglength message_data_length,
 		const struct sockaddr *addr,
 		socklen_t addr_len,
-		int protocol
+		uint8_t protocol
 ) {
 	int ret = 0;
 
@@ -45,11 +46,17 @@ int rrr_msg_holder_util_new_with_empty_message (
 
 	// XXX : Callers treat this function as message_data_length is an absolute value
 
-	ssize_t message_size = sizeof(*message) - 1 + message_data_length;
+	rrr_biglength message_size = sizeof(*message) - 1 + message_data_length;
 
-	message = rrr_allocate(message_size);
-	if (message == NULL) {
+	if (message_size < message_data_length) {
+		RRR_MSG_0("Overflow in while creating message holder with empty message\n");
+		ret = 1;
+		goto out;
+	}
+
+	if ((message = rrr_allocate(message_size)) == NULL) {
 		RRR_MSG_0("Could not allocate message in message_holder_new_with_message\n");
+		ret = 1;
 		goto out;
 	}
 
@@ -67,7 +74,7 @@ int rrr_msg_holder_util_new_with_empty_message (
 	}
 
 	rrr_msg_holder_lock(entry);
-	memset(message, '\0', message_size);
+	rrr_memset(message, '\0', message_size);
 	rrr_msg_holder_unlock(entry);
 
 	message = NULL;
@@ -84,25 +91,66 @@ int rrr_msg_holder_util_clone_no_locking (
 		const struct rrr_msg_holder *source
 ) {
 	// Note : Do calculation correctly, not incorrect
-	ssize_t message_data_length = source->data_length - (sizeof(struct rrr_msg_msg) - 1);
+	rrr_biglength message_data_length = source->data_length - (sizeof(struct rrr_msg_msg) - 1);
 
-	if (message_data_length < 0) {
-		RRR_BUG("Message too small in rrr_msg_msg_holder_clone_no_locking\n");
-	}
+	int ret = 0;
 
-	int ret = rrr_msg_holder_util_new_with_empty_message (
-			result,
+	*result = NULL;
+
+	struct rrr_msg_holder *entry = NULL;
+
+	if ((ret = rrr_msg_holder_util_new_with_empty_message (
+			&entry,
 			message_data_length,
 			(struct sockaddr *) &source->addr,
 			source->addr_len,
 			source->protocol
+	)) != 0) {
+		goto out;
+	}
+
+	rrr_msg_holder_lock(entry);
+
+	entry->buffer_time = source->buffer_time;
+	entry->send_time = source->send_time;
+	rrr_memcpy(entry->message, source->message, source->data_length);
+
+	ret = rrr_instance_friend_collection_append_from (&entry->nexthops, &source->nexthops);
+
+	rrr_msg_holder_unlock(entry);
+
+	if (ret != 0) {
+		goto out;
+	}
+
+	*result = entry;
+	entry = NULL;
+
+	out:
+	if (entry != NULL) {
+		rrr_msg_holder_decref(entry);
+	}
+	return ret;
+}
+
+int rrr_msg_holder_util_clone_no_locking_no_metadata (
+		struct rrr_msg_holder **result,
+		const struct rrr_msg_holder *source
+) {
+	// Note : Do calculation correctly, not incorrect
+	rrr_biglength message_data_length = source->data_length - (sizeof(struct rrr_msg_msg) - 1);
+
+	int ret = rrr_msg_holder_util_new_with_empty_message (
+			result,
+			message_data_length,
+			NULL,
+			0,
+			0
 	);
 
 	if (ret == 0) {
 		rrr_msg_holder_lock(*result);
-		(*result)->buffer_time = source->buffer_time;
-		(*result)->send_time = source->send_time;
-		memcpy((*result)->message, source->message, source->data_length);
+		rrr_memcpy((*result)->message, source->message, source->data_length);
 		rrr_msg_holder_unlock(*result);
 	}
 
