@@ -34,10 +34,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "mqtt_packet.h"
 #include "mqtt_parse.h"
 #include "mqtt_assemble.h"
+#include "mqtt_payload.h"
 
 #include "../ip/ip.h"
 #include "../ip/ip_accept_data.h"
-#include "../buffer.h"
+#include "../fifo.h"
 #include "../net_transport/net_transport.h"
 #include "../rrr_strerror.h"
 #include "../util/macro_utils.h"
@@ -71,13 +72,18 @@ static int __rrr_mqtt_connection_call_event_handler (struct rrr_mqtt_conn *conne
 #define CALL_EVENT_HANDLER_NO_REPEAT(event)	\
 		__rrr_mqtt_connection_call_event_handler(connection, event, 1, NULL)
 
+#define RRR_MQTT_CONN_TIMEOUTS_CHECK()                         \
+        short keepalive_reached = 0;                           \
+        short keepalive_exceeded = 0;                          \
+        __rrr_mqtt_conn_timeouts_check (&keepalive_reached, &keepalive_exceeded, connection)
+
 int rrr_mqtt_conn_set_client_id (
 		struct rrr_mqtt_conn *connection,
 		const char *id
 ) {
 	RRR_FREE_IF_NOT_NULL(connection->client_id);
 	if ((connection->client_id = rrr_strdup(id)) == NULL) {
-		RRR_MSG_0("Could not allocate memory in rrr_mqtt_conn_update_client_id\n");
+		RRR_MSG_0("Could not allocate memory in %s\n", __func__);
 		return 1;
 	}
 	return 0;
@@ -145,12 +151,21 @@ int rrr_mqtt_conn_update_state (
 		RRR_MQTT_CONN_STATE_SET (connection, RRR_MQTT_CONN_STATE_CLOSE_WAIT);
 	}
 	else {
-		RRR_BUG("Unknown control packet %u in rrr_mqtt_connection_update_state_iterator_ctx\n", packet_type);
+		RRR_BUG("Unknown control packet %u in %s\n", packet_type, __func__);
 	}
 
 	return RRR_MQTT_OK;
 }
 
+int rrr_mqtt_conn_iterator_ctx_set_disconnect_reason (
+		struct rrr_net_transport_handle *handle,
+		uint8_t reason_v5
+) {
+	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
+	RRR_MQTT_CONN_SET_DISCONNECT_REASON_IF_ZERO(connection, reason_v5);
+	return RRR_MQTT_OK;
+}
+
 int rrr_mqtt_conn_iterator_ctx_send_disconnect (
 		struct rrr_net_transport_handle *handle
 ) {
@@ -169,7 +184,7 @@ int rrr_mqtt_conn_iterator_ctx_send_disconnect (
 			connection->protocol_version
 	);
 	if (disconnect == NULL) {
-		RRR_MSG_0("Could not allocate DISCONNECT packet in rrr_mqtt_conn_iterator_ctx_send_disconnect\n");
+		RRR_MSG_0("Could not allocate DISCONNECT packet in %s\n", __func__);
 		ret = RRR_MQTT_INTERNAL_ERROR;
 		goto out_no_decref;
 	}
@@ -184,8 +199,8 @@ int rrr_mqtt_conn_iterator_ctx_send_disconnect (
 		)) != RRR_MQTT_OK) {
 			ret = ret & ~RRR_MQTT_SOFT_ERROR;
 			if (ret != RRR_MQTT_OK) {
-				RRR_MSG_0("Error while queuing outbound DISCONNECT packet in rrr_mqtt_conn_iterator_ctx_send_disconnect return was %i\n",
-						ret);
+				RRR_MSG_0("Error while queuing outbound DISCONNECT packet in %s return was %i\n",
+						__func__, ret);
 				goto send_disconnect_out;
 			}
 			ret |= RRR_MQTT_SOFT_ERROR;
@@ -215,13 +230,9 @@ static void __rrr_mqtt_connection_close (
 	RRR_MQTT_CONN_STATE_SET(connection, RRR_MQTT_CONN_STATE_CLOSED);
 }
 
-static void __rrr_mqtt_connection_will_properties_destroy (struct rrr_mqtt_conn_will_properties *will_properties) {
-	rrr_mqtt_property_collection_clear(&will_properties->user_properties);
-}
-
 static void __rrr_mqtt_connection_destroy (struct rrr_mqtt_conn *connection) {
 	if (connection == NULL) {
-		RRR_BUG("NULL pointer in __rrr_mqtt_connection_destroy\n");
+		RRR_BUG("NULL pointer in %s\n", __func__);
 	}
 
 	// This will be cleaned up anyway, it's more for informational purposes
@@ -232,16 +243,13 @@ static void __rrr_mqtt_connection_destroy (struct rrr_mqtt_conn *connection) {
 
 	RRR_DBG_2("Destroying connection %p, final destruction\n", connection);
 
-	rrr_fifo_buffer_clear(&connection->receive_queue.buffer);
+	rrr_fifo_clear(&connection->receive_buffer.buffer);
 
 	rrr_mqtt_parse_session_destroy(&connection->parse_session);
 
 	RRR_FREE_IF_NOT_NULL(connection->client_id);
 	RRR_FREE_IF_NOT_NULL(connection->username);
 
-	RRR_MQTT_P_DECREF_IF_NOT_NULL(connection->will_publish);
-	__rrr_mqtt_connection_will_properties_destroy(&connection->will_properties);
-
 	rrr_free(connection);
 }
 
@@ -263,18 +271,18 @@ static int __rrr_mqtt_conn_new (
 
 	res = rrr_allocate(sizeof(*res));
 	if (res == NULL) {
-		RRR_MSG_0("Could not allocate memory in rrr_mqtt_connection_new\n");
+		RRR_MSG_0("Could not allocate memory in %s\n", __func__);
 		ret = RRR_MQTT_INTERNAL_ERROR;
 		goto out;
 	}
 
 	memset (res, '\0', sizeof(*res));
 
-	if ((ret = rrr_fifo_buffer_init_custom_free(&res->receive_queue.buffer,	rrr_mqtt_p_standardized_decref)) != 0) {
-		RRR_MSG_0("Could not initialize buffers in __rrr_mqtt_connection_new\n");
-		ret = RRR_MQTT_INTERNAL_ERROR;
-		goto out_free;
-	}
+	rrr_fifo_init_custom_refcount (
+			&res->receive_buffer.buffer,
+			rrr_mqtt_p_usercount_incref_void,
+			rrr_mqtt_p_usercount_decref_void
+	);
 
 	res->connect_time = res->last_read_time = res->last_write_time = rrr_time_get_64();
 	res->close_wait_time_usec = close_wait_time_usec;
@@ -297,7 +305,7 @@ static int __rrr_mqtt_conn_new (
 			break;
 		}
 		default: {
-			RRR_BUG("Received non INET/INET6 sockaddr struct in __rrr_mqtt_connection_new\n");
+			RRR_BUG("Received non INET/INET6 sockaddr struct in %s\n", __func__);
 		}
 	}
 
@@ -305,13 +313,37 @@ static int __rrr_mqtt_conn_new (
 
 	goto out;
 
-	out_free:
-		rrr_free(res);
-
+//	out_free:
+//		rrr_free(res);
 	out:
 		return ret;
 }
 
+static void __rrr_mqtt_conn_timeouts_check (
+		short *keepalive_reached,
+		short *keepalive_exceeded,
+		struct rrr_mqtt_conn *connection
+) {
+	*keepalive_reached = 0;
+	*keepalive_exceeded = 0;
+
+	uint64_t limit_ping = connection->keep_alive;
+	uint64_t limit = (uint64_t) ((double) connection->keep_alive * 1.5);
+
+	limit_ping *= 1000000;
+	limit *= 1000000;
+
+	if (connection->last_read_time + limit < rrr_time_get_64()) {
+		*keepalive_exceeded = 1;
+	}
+
+	if ( connection->last_read_time + limit_ping < rrr_time_get_64() ||
+	     connection->last_write_time + limit_ping < rrr_time_get_64()
+	) {
+		*keepalive_reached = 1;
+	}
+}
+
 static int __rrr_mqtt_connection_disconnect_call_event_handler_if_needed (
 		struct rrr_mqtt_conn *connection
 ) {
@@ -344,8 +376,6 @@ static int __rrr_mqtt_connection_in_iterator_disconnect (
 
 	int ret = RRR_MQTT_OK;
 
-//	printf("in iteratore disconnect state: %u\n", connection->state_flags);
-
 	// The session system must be informed (through broker/client event handlers) before close_wait expires
 	// in case the client re-connects before close_wait has finished. This prevents the new connection
 	// from experiencing the session being destroyed shortly after connecting.
@@ -357,24 +387,42 @@ static int __rrr_mqtt_connection_in_iterator_disconnect (
 	// will transition to CLOSE_WAIT. If a disconnect packet is already received or sent otherwise
 	// by non-error means, state is already CLOSE_WAIT here. For severe errors with V3.1 we do not
 	// send the packet, these errors will set a disconnect reason >= 0x80.
-	if (!RRR_MQTT_CONN_STATE_IS_CLOSED_OR_CLOSE_WAIT(connection)) {
+
+	RRR_MQTT_CONN_TIMEOUTS_CHECK();
+
+	if (keepalive_exceeded || connection->disconnect_reason_v5_ == RRR_MQTT_P_5_REASON_KEEP_ALIVE_TIMEOUT) {
+		RRR_DBG_1(">>>X Destroying connection %p client ID '%s', keep alive was exceeded. Reason is otherwise set to %u.\n",
+				connection,
+				(connection->client_id != NULL ? connection->client_id : "(empty)"),
+				connection->disconnect_reason_v5_
+		);
+
+		// Must destroy connection immediately without DISCONNECT packet. There is a reason defined
+		// in v5, but the same standard also instructs to close the connection without
+		// sending any DISCONNECT.
+
+		goto out;
+	}
+	else if (!RRR_MQTT_CONN_STATE_IS_CLOSED_OR_CLOSE_WAIT(connection)) {
 		if (connection->disconnect_reason_v5_ >= 0x80) {
 			const struct rrr_mqtt_p_reason *reason = rrr_mqtt_p_reason_get_v5(connection->disconnect_reason_v5_);
-			RRR_MSG_0("Severe error %u ('%s') for connection with client id '%s', must disconnect now\n",
+			RRR_DBG_1(">>>X Severe error %u ('%s') for connection with client id '%s', must disconnect now\n",
 					connection->disconnect_reason_v5_,
 					(reason != NULL ? reason->description : "unknown error"),
 					(connection->client_id != NULL ? connection->client_id : "")
 			);
 		}
 
-		if ((connection->protocol_version != NULL && connection->protocol_version->id == RRR_MQTT_VERSION_5)
-				|| connection->disconnect_reason_v5_ < 0x80
+		// To trigger any will message when version is not 5, close the connection without sending disconnect packet
+
+		if ( (connection->protocol_version != NULL && connection->protocol_version->id == RRR_MQTT_VERSION_5) ||
+		     (connection->disconnect_reason_v5_ < 0x80 && connection->disconnect_reason_v5_ != RRR_MQTT_P_5_REASON_DISCONNECT_WITH_WILL)
 		) {
 			ret = rrr_mqtt_conn_iterator_ctx_send_disconnect(handle);
 			// Ignore soft errors when sending DISCONNECT packet here.
 			ret &= ~(RRR_MQTT_SOFT_ERROR);
 			if (ret != 0) {
-				RRR_MSG_0("Internal error sending disconnect packet in __rrr_mqtt_connection_collection_in_iterator_disconnect_and_destroy\n");
+				RRR_MSG_0("Internal error sending disconnect packet in %s\n", __func__);
 				goto out;
 			}
 		}
@@ -385,34 +433,42 @@ static int __rrr_mqtt_connection_in_iterator_disconnect (
 		}
 	}
 
-	if (connection->close_wait_time_usec > 0) {
-		uint64_t time_now = rrr_time_get_64();
-		if (connection->close_wait_start == 0) {
-			connection->close_wait_start = time_now;
-			RRR_DBG_1("Destroying connection %p client ID '%s' reason %u, starting timer (and closing connection if needed).\n",
-					connection,
-					(connection->client_id != NULL ? connection->client_id : "(empty)"),
-					connection->disconnect_reason_v5_
-			);
+	if (connection->close_wait_time_usec == 0) {
+		goto out;
+	}
 
-			if (!RRR_MQTT_CONN_STATE_IS_CLOSED(connection)) {
-				__rrr_mqtt_connection_close (connection);
-			}
-		}
-		if (time_now - connection->close_wait_start < connection->close_wait_time_usec) {
-/*			printf ("Connection is not to be closed closed yet, waiting %" PRIu64 " usecs\n",
-					(*cur)->close_wait_time_usec - (time_now - (*cur)->close_wait_start));*/
-			// We can basically return anything apart from 1 to stop net transport from destroying connection.
-			// When we return 0, connection is destroyed.
-			ret = RRR_LL_DIDNT_DESTROY;
-			goto out;
-		}
-		RRR_DBG_2("Destroying connection %p reason %u, timer done\n",
-				connection, connection->disconnect_reason_v5_);
+	const uint64_t time_now = rrr_time_get_64();
+
+	if (connection->close_wait_start == 0) {
+		connection->close_wait_start = time_now;
+
+		RRR_DBG_1(">>>X Destroying connection %p client ID '%s' reason %u, starting timer %" PRIu64 " usecs (and closing connection if needed).\n",
+				connection,
+				(connection->client_id != NULL ? connection->client_id : "(empty)"),
+				connection->disconnect_reason_v5_,
+				connection->close_wait_time_usec
+		);
+
+	}
+
+	if (connection->close_wait_start + connection->close_wait_time_usec > time_now) {
+//			printf ("Connection is not to be closed closed yet, waiting %" PRIu64 " usecs\n",
+//				connection->close_wait_time_usec - (time_now - connection->close_wait_start)).
+
+		// When we return EOF, connection is not destroyed.
+		ret = RRR_NET_TRANSPORT_READ_READ_EOF;
+		goto out;
 	}
 
 	out:
-	return ret;
+		if (ret != RRR_NET_TRANSPORT_READ_READ_EOF) {
+			if (!RRR_MQTT_CONN_STATE_IS_CLOSED(connection)) {
+				__rrr_mqtt_connection_close (connection);
+			}
+			RRR_DBG_2("Destroying connection %p reason %u, timer done\n",
+					connection, connection->disconnect_reason_v5_);
+		}
+		return ret;
 }
 
 int rrr_mqtt_conn_set_data_from_connect_and_connack (
@@ -431,7 +487,7 @@ int rrr_mqtt_conn_set_data_from_connect_and_connack (
 	if (username != NULL && *username != '\0') {
 		RRR_FREE_IF_NOT_NULL(connection->username);
 		if ((connection->username = rrr_strdup(username)) == NULL) {
-			RRR_MSG_0("Could not allocate memory for username in rrr_mqtt_conn_iterator_ctx_set_data_from_connect\n");
+			RRR_MSG_0("Could not allocate memory for username in %s\n", __func__);
 			ret = RRR_MQTT_INTERNAL_ERROR;
 		}
 	}
@@ -439,87 +495,6 @@ int rrr_mqtt_conn_set_data_from_connect_and_connack (
 	return ret;
 }
 
-int rrr_mqtt_conn_set_will_data_from_connect (
-		uint8_t *reason_v5,
-		struct rrr_mqtt_conn *connection,
-		const struct rrr_mqtt_p_connect *connect
-) {
-	int ret = 0;
-
-	struct rrr_mqtt_p_publish *publish = NULL;
-
-	RRR_DBG_3("Set will message for client '%s' with topic '%s' retain '%u' qos '%u' in MQTT broker\n",
-			connection->client_id,
-			connect->will_topic,
-			RRR_MQTT_P_CONNECT_GET_FLAG_WILL_RETAIN(connect),
-			RRR_MQTT_P_CONNECT_GET_FLAG_WILL_QOS(connect)
-	);
-
-	struct rrr_mqtt_common_parse_will_properties_callback_data callback_data = {
-			&connect->will_properties,
-			0,
-			&connection->will_properties
-	};
-
-	if ((ret = rrr_mqtt_property_collection_iterate (
-			&connect->will_properties,
-			rrr_mqtt_common_parse_will_properties_callback,
-			&callback_data
-	)) != 0) {
-		*reason_v5 = callback_data.reason_v5;
-		if (ret != RRR_MQTT_SOFT_ERROR) {
-			RRR_MSG_0("Hard error while iterating will properties in rrr_mqtt_conn_set_will_data_from_connect\n");
-		}
-		goto out;
-	}
-
-	if (rrr_mqtt_p_new_publish (
-			&publish,
-			connect->will_topic,
-			connect->will_message,
-			connect->will_message_size,
-			connect->protocol_version
-	) != 0) {
-		RRR_MSG_0("Could not allocate publish in rrr_mqtt_conn_set_will_data_from_connect\n");
-		ret = 1;
-		goto out;
-	}
-
-	// These fields are present in both CONNECT will properties and PUBLISH properties. They
-	// are copied directly to the new will PUBLISH.
-	uint8_t will_property_list[] = {
-			RRR_MQTT_PROPERTY_PAYLOAD_FORMAT_INDICATOR,
-			RRR_MQTT_PROPERTY_MESSAGE_EXPIRY_INTERVAL,
-			RRR_MQTT_PROPERTY_CONTENT_TYPE,
-			RRR_MQTT_PROPERTY_RESPONSE_TOPIC,
-			RRR_MQTT_PROPERTY_CORRELATION_DATA,
-			RRR_MQTT_PROPERTY_USER_PROPERTY
-	};
-
-	RRR_MQTT_P_PUBLISH_SET_FLAG_QOS(publish, RRR_MQTT_P_CONNECT_GET_FLAG_WILL_QOS(connect));
-	RRR_MQTT_P_PUBLISH_SET_FLAG_RETAIN(publish, RRR_MQTT_P_CONNECT_GET_FLAG_WILL_RETAIN(connect));
-
-	publish->will_delay_interval = connection->will_properties.will_delay_interval;
-
-	if (rrr_mqtt_property_collection_add_selected_from_collection (
-			&publish->properties,
-			&connect->will_properties,
-			will_property_list,
-			sizeof(will_property_list)/sizeof(*will_property_list)
-	) != 0) {
-		RRR_MSG_0("Error while copying will properties to publish in rrr_mqtt_conn_set_will_data_from_connect\n");
-		ret = 1;
-		goto out;
-	}
-
-	connection->will_publish = publish;
-	publish = NULL;
-
-	out:
-	RRR_MQTT_P_DECREF_IF_NOT_NULL(publish);
-	return ret;
-}
-
 int rrr_mqtt_conn_iterator_ctx_housekeeping (
 		struct rrr_net_transport_handle *handle,
 		int (*exceeded_keep_alive_callback)(struct rrr_net_transport_handle *handle, void *arg),
@@ -530,22 +505,19 @@ int rrr_mqtt_conn_iterator_ctx_housekeeping (
 	int ret = RRR_MQTT_OK;
 
 	if (connection->keep_alive > 0) {
-		uint64_t limit_ping = (double) connection->keep_alive;
-		uint64_t limit = (double) connection->keep_alive * 1.5;
+		RRR_MQTT_CONN_TIMEOUTS_CHECK();
 
-		limit_ping *= 1000000;
-		limit *= 1000000;
-		if (connection->last_read_time + limit < rrr_time_get_64()) {
+		if (keepalive_exceeded) {
 			RRR_DBG_1("Keep-alive exceeded for connection\n");
+			RRR_MQTT_CONN_SET_DISCONNECT_REASON_IF_ZERO(connection, RRR_MQTT_P_5_REASON_KEEP_ALIVE_TIMEOUT);
 			ret = RRR_MQTT_SOFT_ERROR;
 			goto out;
 		}
-		else if (exceeded_keep_alive_callback != NULL &&
-				(connection->last_read_time + limit_ping < rrr_time_get_64() ||
-				connection->last_write_time + limit_ping < rrr_time_get_64()) &&
-				(ret = exceeded_keep_alive_callback(handle, callback_arg)) != RRR_MQTT_OK
+		else if ((keepalive_reached) &&
+		         (exceeded_keep_alive_callback) != NULL &&
+                         (ret = exceeded_keep_alive_callback(handle, callback_arg)) != RRR_MQTT_OK
 		) {
-			RRR_MSG_0("Error from callback in rrr_mqtt_conn_iterator_ctx_housekeeping after exceeded keep-alive\n");
+			RRR_MSG_0("Error from callback in %s after exceeded keep-alive\n", __func__);
 			goto out;
 		}
 	}
@@ -573,7 +545,7 @@ void rrr_mqtt_conn_accept_and_connect_callback (
 			callback_data->event_handler,
 			callback_data->event_handler_arg
 	) != RRR_MQTT_OK) {
-		RRR_MSG_0("Could not create connection in __rrr_mqtt_conn_collection_accept_connect_callback\n");
+		RRR_MSG_0("Could not create connection in %s\n", __func__);
 		goto out;
 	}
 
@@ -589,6 +561,7 @@ void rrr_mqtt_conn_accept_and_connect_callback (
 int rrr_mqtt_conn_iterator_ctx_check_alive (
 		int *alive,
 		int *send_allowed,
+		int *close_wait,
 		struct rrr_net_transport_handle *handle
 ) {
 	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
@@ -596,6 +569,8 @@ int rrr_mqtt_conn_iterator_ctx_check_alive (
 	int ret = RRR_MQTT_OK;
 
 	*alive = 1;
+	*send_allowed = 0;
+	*close_wait = 0;
 
 	if (RRR_MQTT_CONN_STATE_IS_CLOSED_OR_CLOSE_WAIT(connection) ||
 		RRR_MQTT_CONN_STATE_IS_CLOSED(connection) ||
@@ -608,6 +583,11 @@ int rrr_mqtt_conn_iterator_ctx_check_alive (
 		*send_allowed = 1;
 	}
 
+
+	if (RRR_MQTT_CONN_STATE_IS_CLOSE_WAIT(connection)) {
+		*close_wait = 1;
+	}
+
 	return ret;
 }
 
@@ -666,11 +646,6 @@ struct rrr_mqtt_conn_read_callback_data {
 	void *handler_callback_arg;
 };
 
-#define RRR_MQTT_CONN_SET_DISCONNECT_REASON_IF_ZERO(connection, reason) 	\
-		if ((connection)->disconnect_reason_v5_ == 0) {						\
-			(connection)->disconnect_reason_v5_ = reason;					\
-		}
-
 static int __rrr_mqtt_conn_read_get_target_size (
 		struct rrr_read_session *read_session,
 		void *arg
@@ -689,7 +664,7 @@ static int __rrr_mqtt_conn_read_get_target_size (
 				RRR_MQTT_CONN_SET_DISCONNECT_REASON_IF_ZERO(connection, RRR_MQTT_P_5_REASON_MALFORMED_PACKET);
 			}
 			ret &= ~(RRR_SOCKET_READ_INCOMPLETE);
-			RRR_MSG_0("Returned error from __rrr_mqtt_conn_parse: %i\n", ret);
+			RRR_MSG_0("Returned error from prase function in %s: %i\n", __func__, ret);
 			goto out;
 		}
 		// Don't got out, fixed header might be done
@@ -707,6 +682,20 @@ static int __rrr_mqtt_conn_read_get_target_size (
 	return ret;
 }
 
+static void __rrr_mqtt_conn_get_target_size_error (
+		struct rrr_read_session *read_session,
+		int is_hard_err,
+		void *arg
+) {
+	struct rrr_mqtt_conn_read_callback_data *callback_data = arg;
+
+	(void)(read_session);
+	(void)(is_hard_err);
+	(void)(callback_data);
+
+	// Any error messsage goes here
+}
+
 static int __rrr_mqtt_conn_read_complete_callback (
 		struct rrr_read_session *read_session,
 		void *arg
@@ -740,33 +729,33 @@ static int __rrr_mqtt_conn_read_complete_callback (
 
 	if (RRR_MQTT_PARSE_STATUS_IS_MOVE_PAYLOAD_TO_PACKET(&connection->parse_session)) {
 		if (connection->parse_session.packet->payload != NULL) {
-			RRR_BUG("payload data was not NULL in __rrr_mqtt_conn_read_complete_callback while moving payload\n");
+			RRR_BUG("payload data was not NULL while moving payload in %s\n", __func__);
 		}
 
 		ret = rrr_mqtt_p_payload_new_with_allocated_payload (
 				&connection->parse_session.packet->payload,
 				&read_session->rx_buf_ptr, // Set to NULL if success
 				read_session->rx_buf_ptr + connection->parse_session.payload_pos,
-				read_session->rx_buf_wpos - connection->parse_session.payload_pos
+				rrr_length_from_biglength_sub_bug_const (read_session->rx_buf_wpos, connection->parse_session.payload_pos)
 		);
 
 		RRR_FREE_IF_NOT_NULL(read_session->rx_buf_ptr);
 		read_session->rx_buf_wpos = 0;
 
 		if (ret != 0) {
-			RRR_MSG_0("Could not move payload to packet in __rrr_mqtt_conn_read_complete_callback\n");
+			RRR_MSG_0("Could not move payload to packet in %s\n", __func__);
 			goto out;
 		}
 	}
 
 	if ((ret = CALL_EVENT_HANDLER_ARG(RRR_MQTT_CONN_EVENT_PACKET_PARSED, connection->parse_session.packet)) != 0) {
-		RRR_MSG_0("Error from event handler in __rrr_mqtt_conn_read_complete_callback, return was %i\n", ret);
+		RRR_MSG_0("Error from event handler in %s, return was %i\n", __func__, ret);
 		goto out;
 	}
 
 	rrr_mqtt_packet_parse_session_extract_packet(&packet, &connection->parse_session);
 
-	if (rrr_mqtt_p_standardized_get_refcount(packet) != 1) {
+	if (RRR_MQTT_P_USERCOUNT(packet) != 1) {
 		RRR_BUG("Refcount was not 1 while finalizing mqtt packet and adding to receive buffer\n");
 	}
 
@@ -791,8 +780,7 @@ static int __rrr_mqtt_conn_read_complete_callback (
 
 int rrr_mqtt_conn_iterator_ctx_read (
 		struct rrr_net_transport_handle *handle,
-		int read_step_max_size,
-		int read_per_round_max,
+		rrr_biglength read_step_max_size,
 		int (*handler_callback) (
 				struct rrr_net_transport_handle *handle,
 				struct rrr_mqtt_p *packet,
@@ -810,29 +798,28 @@ int rrr_mqtt_conn_iterator_ctx_read (
 			handler_callback_arg
 	};
 
-	do {
-		if ((ret = rrr_net_transport_ctx_read_message (
-				handle,
-				2, // Read two times this round
-				2, // Read only two bytes the first time
-				read_step_max_size,
-				0, // No max read size
-				100 * 1000, // 100 ms ratelimit interval
-				(1 * 1024 * 1024) / 20, // 1/20 MB
-				__rrr_mqtt_conn_read_get_target_size,
-				&callback_data,
-				__rrr_mqtt_conn_read_complete_callback,
-				&callback_data
-		)) != 0) {
-			if (ret == RRR_NET_TRANSPORT_READ_RATELIMIT) {
-				ret = RRR_SOCKET_READ_INCOMPLETE;
-			}
-			else if (ret != RRR_SOCKET_READ_INCOMPLETE && connection->disconnect_reason_v5_ == 0) {
-				connection->disconnect_reason_v5_ = RRR_MQTT_P_5_REASON_UNSPECIFIED_ERROR;
-			}
-			goto out;
+	if ((ret = rrr_net_transport_ctx_read_message (
+			handle,
+			2, // Read only two bytes the first time
+			read_step_max_size,
+			0, // No max read size
+			100 * 1000, // 100 ms ratelimit interval
+			(1 * 1024 * 1024) / 20, // 1/20 MB
+			__rrr_mqtt_conn_read_get_target_size,
+			&callback_data,
+			__rrr_mqtt_conn_get_target_size_error,
+			&callback_data,
+			__rrr_mqtt_conn_read_complete_callback,
+			&callback_data
+	)) != 0) {
+		if (ret == RRR_NET_TRANSPORT_READ_RATELIMIT) {
+			ret = RRR_SOCKET_READ_INCOMPLETE;
 		}
-	} while (--read_per_round_max && ret == 0);
+		else if (ret != RRR_SOCKET_READ_INCOMPLETE) {
+			RRR_MQTT_CONN_SET_DISCONNECT_REASON_IF_ZERO(connection, RRR_MQTT_P_5_REASON_UNSPECIFIED_ERROR);
+		}
+		goto out;
+	}
 
 	out:
 	return ret;
@@ -841,14 +828,14 @@ int rrr_mqtt_conn_iterator_ctx_read (
 static int __rrr_mqtt_conn_iterator_ctx_send_push (
 		struct rrr_net_transport_handle *handle,
 		void **data,
-		ssize_t data_size
+		rrr_length data_size
 ) {
 	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
 
 	int ret = 0;
 
 	if ((ret = rrr_net_transport_ctx_send_push(handle, data, data_size)) != 0) {
-		RRR_MSG_0("Error while pushing packet to send queue in __rrr_mqtt_conn_iterator_ctx_write\n");
+		RRR_MSG_0("Error while pushing packet to send queue in %s\n", __func__);
 		ret = RRR_MQTT_SOFT_ERROR;
 		goto out;
 	}
@@ -860,14 +847,14 @@ static int __rrr_mqtt_conn_iterator_ctx_send_push (
 static int __rrr_mqtt_conn_iterator_ctx_send_push_urgent (
 		struct rrr_net_transport_handle *handle,
 		void **data,
-		ssize_t data_size
+		rrr_length data_size
 ) {
 	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
 
 	int ret = 0;
 
 	if ((ret = rrr_net_transport_ctx_send_push_urgent(handle, data, data_size)) != 0) {
-		RRR_MSG_0("Error while sending packet in __rrr_mqtt_conn_iterator_ctx_send_urgent\n");
+		RRR_MSG_0("Error while sending packet in %s\n", __func__);
 		ret = RRR_MQTT_SOFT_ERROR;
 		goto out;
 	}
@@ -878,13 +865,13 @@ static int __rrr_mqtt_conn_iterator_ctx_send_push_urgent (
 
 int __rrr_mqtt_connection_create_variable_int (
 		uint8_t *target,
-		ssize_t *length,
-		uint32_t value
+		rrr_length *length,
+		rrr_biglength value
 ) {
 	*length = 1;
 
-	if (value > 0xfffffff) {
-		RRR_MSG_0("Integer value too large in __rrr_mqtt_connection_create_variable_int\n");
+	if (value > 0xfffffff) { // Seven f's
+		RRR_MSG_0("Integer value too large in %s\n", __func__);
 		return RRR_MQTT_INTERNAL_ERROR;
 	}
 
@@ -910,15 +897,14 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 	RRR_MQTT_DEFINE_CONN_FROM_HANDLE_AND_CHECK;
 
 	int ret = RRR_MQTT_OK;
-	int ret_destroy = 0;
 
 	struct rrr_mqtt_p_payload *payload = NULL;
 	char *network_data = NULL;
-	ssize_t network_size = 0;
+	rrr_length network_size = 0;
 	void *send_data = NULL;
 
 	if (!RRR_MQTT_CONN_STATE_SEND_ANY_IS_ALLOWED(connection) && RRR_MQTT_P_GET_TYPE(packet) == RRR_MQTT_P_TYPE_PUBLISH) {
-		RRR_BUG("BUG: Tried to send PUBLISH while not allowed in __rrr_mqtt_conn_iterator_ctx_send_packet\n");
+		RRR_BUG("BUG: Tried to send PUBLISH while not allowed in %s\n", __func__);
 	}
 
 	// Packets which originate from other hosts might have different protocol
@@ -939,8 +925,8 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 		);
 
 		if (network_data == NULL) {
-			RRR_BUG("Assembled packet of type %s was NULL in rrr_mqtt_conn_iterator_ctx_send_packet\n",
-					RRR_MQTT_P_GET_TYPE_NAME(packet));
+			RRR_BUG("Assembled packet of type %s was NULL in %s\n",
+					RRR_MQTT_P_GET_TYPE_NAME(packet), __func__);
 		}
 
 		if (network_size == 0) {
@@ -955,12 +941,12 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 
 		if (ret_tmp != RRR_MQTT_ASSEMBLE_OK) {
 			if (ret_tmp == RRR_MQTT_ASSEMBLE_INTERNAL_ERR) {
-				RRR_MSG_0("Error while assembling packet in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+				RRR_MSG_0("Error while assembling packet in %s\n", __func__);
 				ret = RRR_MQTT_INTERNAL_ERROR;
 				goto out;
 			}
 			else {
-				RRR_BUG("Unknown return value %i from assembler in rrr_mqtt_conn_iterator_ctx_send_packet\n", ret_tmp);
+				RRR_BUG("Unknown return value %i from assembler in %s\n", ret_tmp, __func__);
 			}
 		}
 	}
@@ -972,36 +958,36 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 	if (RRR_MQTT_P_IS_RESERVED_FLAGS(packet) &&
 		RRR_MQTT_P_GET_PROP_FLAGS(packet) != RRR_MQTT_P_GET_TYPE_FLAGS(packet)
 	) {
-		RRR_BUG("Illegal flags %u for packet type %s in rrr_mqtt_conn_iterator_ctx_send_packet\n",
-				RRR_MQTT_P_GET_TYPE_FLAGS(packet), RRR_MQTT_P_GET_TYPE_NAME(packet));
+		RRR_BUG("Illegal flags %u for packet type %s in %s\n",
+				RRR_MQTT_P_GET_TYPE_FLAGS(packet), RRR_MQTT_P_GET_TYPE_NAME(packet), __func__);
 	}
 
 	struct rrr_mqtt_p_header header = {0};
-	ssize_t variable_int_length = 0;
-	ssize_t payload_length = 0;
+	rrr_length variable_int_length = 0;
+	rrr_length payload_size = 0;
 	payload = packet->payload;
 	if (payload != NULL) {
-		payload_length = packet->payload->length;
+		payload_size = packet->payload->size;
 	}
 
 	if ((ret = __rrr_mqtt_connection_create_variable_int (
 			header.length,
 			&variable_int_length,
-			packet->assembled_data_size + payload_length
+			rrr_length_add_bug_const(packet->assembled_data_size, payload_size)
 	)) != 0) {
-		RRR_MSG_0("Error while creating variable int in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+		RRR_MSG_0("Error while creating variable int in %s\n", __func__);
 		goto out;
 	}
-	header.type = RRR_MQTT_P_GET_TYPE_AND_FLAGS(packet);
+	header.type = (uint8_t) RRR_MQTT_P_GET_TYPE_AND_FLAGS(packet);
 
-	ssize_t total_size = 1 + variable_int_length + packet->assembled_data_size + payload_length;
+	rrr_biglength total_size = 1 + variable_int_length + packet->assembled_data_size + payload_size;
 
-	RRR_DBG_3("Sending packet %p of type %s flen: 1, vlen: %li, alen: %li, plen: %li, total: %li, id: %u, urgent: %i\n",
+	RRR_DBG_3("Sending packet %p of type %s flen: 1, vlen: %" PRIrrrl ", alen: %" PRIrrrl ", plen: %" PRIrrrl ", total: %" PRIrrrbl ", id: %u, urgent: %i\n",
 			packet,
 			RRR_MQTT_P_GET_TYPE_NAME(packet),
 			variable_int_length,
 			packet->assembled_data_size,
-			payload_length,
+			payload_size,
 			total_size,
 			RRR_MQTT_P_GET_IDENTIFIER(packet),
 			urgent
@@ -1009,9 +995,14 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 
 	__rrr_mqtt_connection_update_last_write_time(connection);
 
-	const size_t send_size = sizeof(header.type) + variable_int_length + packet->assembled_data_size + (payload != NULL ? payload->length : 0);
+	const rrr_biglength send_size = sizeof(header.type) + variable_int_length + packet->assembled_data_size + (payload != NULL ? payload->size : 0);
+	if (send_size > RRR_LENGTH_MAX) {
+		RRR_BUG("Bug: Send size overflow in rrr_mqtt_conn_iterator_ctx_send_packet (%llu>%llu)\n",
+			(unsigned long long) send_size, (unsigned long long) RRR_LENGTH_MAX);
+	}
+
 	if ((send_data = rrr_allocate(send_size)) == NULL) {
-		RRR_MSG_0("Failed to allocate send data in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+		RRR_MSG_0("Failed to allocate send data in %s\n", __func__);
 		ret = 1;
 		goto out;
 	}
@@ -1026,27 +1017,27 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 		send_data_pos += packet->assembled_data_size;
 	}
 	else if (payload != NULL) {
-		RRR_BUG("Payload was present without variable header in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+		RRR_BUG("Payload was present without variable header in %s\n", __func__);
 	}
 
 	if (payload != NULL) {
-		if (payload_length == 0) {
-			RRR_BUG("Payload size was 0 but payload pointer was not NULL in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+		if (payload_size == 0) {
+			RRR_BUG("Payload size was 0 but payload pointer was not NULL in %s\n", __func__);
 		}
-		memcpy(send_data_pos, payload->payload_start, payload->length);
+		memcpy(send_data_pos, payload->payload_start, payload->size);
 	}
 
 	int (*send_method)(
 			struct rrr_net_transport_handle *handle,
 			void **data,
-			ssize_t data_size
+			rrr_length data_size
 	) = (urgent
 		? __rrr_mqtt_conn_iterator_ctx_send_push_urgent
 		: __rrr_mqtt_conn_iterator_ctx_send_push
 	);
 
-	if ((ret = send_method (handle, &send_data, send_size)) != 0) {
-		RRR_MSG_0("Error while pushing data in rrr_mqtt_conn_iterator_ctx_send_packet\n");
+	if ((ret = send_method (handle, &send_data, (rrr_length) send_size)) != 0) {
+		RRR_MSG_0("Error while pushing data in %s\n", __func__);
 		goto out;
 	}
 
@@ -1056,7 +1047,7 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 			RRR_MQTT_CONN_UPDATE_STATE_DIRECTION_OUT
 	);
 	if (ret != RRR_MQTT_OK) {
-		RRR_MSG_0("Could not update connection state in rrr_mqtt_connection_iterator_ctx_send_packet\n");
+		RRR_MSG_0("Could not update connection state in %s\n", __func__);
 		goto out;
 	}
 
@@ -1069,7 +1060,7 @@ static int __rrr_mqtt_conn_iterator_ctx_send_packet (
 	out:
 	RRR_FREE_IF_NOT_NULL(send_data);
 	RRR_FREE_IF_NOT_NULL(network_data);
-	return ret | ret_destroy;
+	return ret;
 }
 
 int rrr_mqtt_conn_iterator_ctx_send_packet (
