@@ -30,7 +30,7 @@ struct rrr_python3_array_data;
 struct rrr_python3_array_value_data;
 
 //int rrr_python3_array_value_count (struct rrr_python3_array_value_data *data);
-int rrr_python3_array_count (struct rrr_python3_array_data *data);
+Py_ssize_t rrr_python3_array_count (struct rrr_python3_array_data *data);
 int rrr_python3_array_check (PyObject *object);
 int rrr_python3_array_value_check (PyObject *object);
 
