@@ -35,6 +35,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "util/macro_utils.h"
 #include "util/slow_noop.h"
 #include "util/gnu.h"
+#include "util/posix.h"
 #include "allocator.h"
 #include "cmdlineparser/cmdline.h"
 #include "threads.h"
@@ -519,8 +520,7 @@ static int __rrr_thread_wait_for_state_initialized (
 		if (state == RRR_THREAD_STATE_RUNNING_FORKED) {
 			RRR_BUG("BUG: Thread %p name %s started prior to receiving signal\n", thread, thread->name);
 		}
-		else if ( state == RRR_THREAD_STATE_NEW ||
-		          state == RRR_THREAD_STATE_INITIALIZED ||
+		else if ( state == RRR_THREAD_STATE_INITIALIZED ||
 		          state == RRR_THREAD_STATE_STOPPED
 		) {
 			was_ok = 1;
@@ -558,13 +558,9 @@ static int __rrr_thread_wait_for_state_forked (
 	//   existing thread structures. Don't do validation on the value, only check with ==
 	//   if it gets the value we want.
 
-	// We can only use the sleep() function which only has second resolution. We therefore
-	// busy-wait for many rounds before finally sleeping if the thread is slow to change
-	// state.
-
 	int was_ok = 0;
 
-	const unsigned long long max = 100000000; // 100 mill
+	const unsigned long long max = 100; // ~ 5 seconds
 	unsigned long long int j;
 	for (j = 0; j <= max; j++)  {
 		if ( thread->state == RRR_THREAD_STATE_RUNNING_FORKED ||
@@ -573,10 +569,11 @@ static int __rrr_thread_wait_for_state_forked (
 			was_ok = 1;
 			break;
 		}
-		if (j > max - 3) {
-			sleep(1);
+		if (j > max / 2) {
+			rrr_posix_msleep_signal_safe (100); // 100 ms
 		}
-		rrr_slow_noop();
+
+		rrr_posix_msleep_signal_safe (25); // 25 ms
 	}
 
 	if (was_ok != 1) {
