@@ -127,7 +127,7 @@ struct file_data {
 	rrr_setting_uint timeout_s;
 
 	char *topic;
-	size_t topic_len;
+	uint16_t topic_len;
 
 	uint64_t message_count;
 
@@ -231,17 +231,7 @@ static int file_parse_config (struct file_data *data, struct rrr_instance_config
 
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_UNSIGNED("file_probe_interval_ms", probe_interval, RRR_FILE_DEFAULT_PROBE_INTERVAL_MS);
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_UTF8_DEFAULT_NULL("file_prefix", prefix);
-
-	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_UTF8_DEFAULT_NULL("file_topic", topic);
-	if (data->topic != NULL) {
-		data->topic_len = strlen(data->topic);
-		if (rrr_mqtt_topic_validate_name(data->topic) != 0) {
-			RRR_MSG_0("Validation of parameter file_topic with value '%s' failed in file instance %s\n",
-					data->topic, config->name);
-			ret = 1;
-			goto out;
-		}
-	}
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_TOPIC("file_topic", topic, topic_len);
 
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_UTF8_DEFAULT_NULL("file_directory", directory);
 	if (data->directory == NULL) {
@@ -574,7 +564,7 @@ static int file_read_array_write_callback (struct rrr_msg_holder *entry, void *a
 
 	uint64_t time = rrr_time_get_64();
 
-	if ((ret = rrr_array_new_message_from_collection (
+	if ((ret = rrr_array_new_message_from_array (
 			&new_message,
 			callback_data->array_final,
 			time,
@@ -624,6 +614,7 @@ static int file_read_array_callback (struct rrr_read_session *read_session, stru
 			NULL,
 			0,
 			0,
+			NULL,
 			file_read_array_write_callback,
 			&write_callback_data,
 			INSTANCE_D_CANCEL_CHECK_ARGS(callback_data->file_data->thread_data)
@@ -660,6 +651,23 @@ static int file_read_all_to_message_get_target_size_callback (
 	return RRR_READ_OK;
 }
 
+static int file_verify_wpos (
+		struct file_data *file_data,
+		const struct rrr_read_session *read_session
+) {
+	if (read_session->rx_buf_wpos > RRR_LENGTH_MAX) {
+		RRR_MSG_0("Too many bytes read in file instance %s (%" PRIrrrbl ">%llu)",
+			INSTANCE_D_NAME(file_data->thread_data),
+			read_session->rx_buf_wpos,
+			(unsigned long long) RRR_LENGTH_MAX
+		);
+		return 1;
+	}
+	return 0;
+}
+
+#define VERIFY_WPOS() do{ if((ret = file_verify_wpos(file_data, read_session)) != 0) goto out; } while(0)
+
 struct file_read_all_to_message_write_callback_data {
 	struct file_data *file_data;
 	struct file *file;
@@ -673,6 +681,8 @@ static int file_read_all_to_message_write_callback_simple (
 ) {
 	int ret = 0;
 
+	VERIFY_WPOS();
+
 	uint64_t time = rrr_time_get_64();
 
 	struct rrr_msg_msg *reading = NULL;
@@ -682,7 +692,7 @@ static int file_read_all_to_message_write_callback_simple (
 			MSG_CLASS_DATA,
 			time,
 			file_data->topic_len,
-			read_session->rx_buf_wpos
+			rrr_length_from_biglength_bug_const(read_session->rx_buf_wpos)
 	)) != 0) {
 		RRR_MSG_0("Could not create message in file_read_all_to_message_write_callback_simple\n");
 		goto out;
@@ -692,12 +702,12 @@ static int file_read_all_to_message_write_callback_simple (
 		memcpy(MSG_TOPIC_PTR(reading), file_data->topic, file_data->topic_len);
 	}
 
-	memcpy(MSG_DATA_PTR(reading), read_session->rx_buf_ptr, read_session->rx_buf_wpos);
+	rrr_memcpy(MSG_DATA_PTR(reading), read_session->rx_buf_ptr, read_session->rx_buf_wpos);
 
 	entry->message = reading;
 	entry->data_length = MSG_TOTAL_SIZE(reading);
 
-	RRR_DBG_2("file instance %s created message with raw file_data of size %lu and timestamp %" PRIu64 "\n",
+	RRR_DBG_2("file instance %s created message with raw file_data of size %" PRIrrrbl " and timestamp %" PRIu64 "\n",
 			INSTANCE_D_NAME(file_data->thread_data), read_session->rx_buf_wpos, time);
 
 	out:
@@ -714,8 +724,13 @@ static int file_read_all_to_message_write_callback_structured (
 
 	struct rrr_array array_tmp = {0};
 
+	VERIFY_WPOS();
+
 	if ((ret = rrr_array_push_value_blob_with_tag_with_size (
-			&array_tmp, "data", read_session->rx_buf_ptr, read_session->rx_buf_wpos
+			&array_tmp,
+			"data",
+			read_session->rx_buf_ptr,
+			rrr_length_from_biglength_bug_const(read_session->rx_buf_wpos)
 	)) != 0) {
 		RRR_MSG_0("Failed to push file data to array in file_read_all_to_message_write_callback_structured\n");
 		goto out;
@@ -766,7 +781,7 @@ static int file_read_all_to_message_write_callback_structured (
 	uint64_t time = rrr_time_get_64();
 
 	struct rrr_msg_msg *reading = NULL;
-	if ((ret = rrr_array_new_message_from_collection (
+	if ((ret = rrr_array_new_message_from_array (
 			&reading,
 			&array_tmp,
 			time,
@@ -780,7 +795,7 @@ static int file_read_all_to_message_write_callback_structured (
 	entry->message = reading;
 	entry->data_length = MSG_TOTAL_SIZE(reading);
 
-	RRR_DBG_2("file instance %s created message with structured raw file_data of size %lu and timestamp %" PRIu64 "\n",
+	RRR_DBG_2("file instance %s created message with structured raw file_data of size %" PRIrrrbl " and timestamp %" PRIu64 "\n",
 			INSTANCE_D_NAME(file_data->thread_data), read_session->rx_buf_wpos, time);
 
 	out:
@@ -838,6 +853,7 @@ static int file_read_all_to_message_complete_callback (
 			NULL,
 			0,
 			0,
+			NULL,
 			file_read_all_to_message_write_callback,
 			&write_callback_data,
 			INSTANCE_D_CANCEL_CHECK_ARGS(data->thread_data)
@@ -908,6 +924,7 @@ static int file_read (uint64_t *bytes_read, struct file_data *data, struct file
 				0, // No ratelimit max bytes
 				RRR_FILE_MAX_SIZE_MB * 1024 * 1024,
 				file_read_array_callback,
+				NULL,
 				&read_callback_data
 		)) != 0) {
 			if (ret == RRR_READ_INCOMPLETE) {
@@ -940,6 +957,8 @@ static int file_read (uint64_t *bytes_read, struct file_data *data, struct file
 				0, // No ratelimit max bytes
 				file_read_all_to_message_get_target_size_callback,
 				NULL,
+				NULL,
+				NULL,
 				file_read_all_to_message_complete_callback,
 				&read_callback_data
 		)) != 0) {
@@ -1029,7 +1048,7 @@ static void *thread_entry_file (struct rrr_thread *thread) {
 	uint64_t bytes_read_accumulator = 0;
 	uint64_t messages_count_prev = 0;
 
-	int ticks = 0;
+	uint64_t ticks = 0;
 	uint64_t messages_count_prev_stats = 0;
 
 	int consecutive_nothing_happened = 0;
