@@ -38,6 +38,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "lib/cmdlineparser/cmdline.h"
 #include "lib/array_tree.h"
 #include "lib/map.h"
+#include "lib/rrr_types.h"
 #include "lib/event/event.h"
 #include "lib/event/event_collection.h"
 #include "lib/messages/msg_msg.h"
@@ -52,6 +53,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "lib/rrr_strerror.h"
 #include "lib/util/rrr_time.h"
 #include "lib/util/posix.h"
+#include "lib/util/arguments.h"
 
 #define RRR_HTTP_CLIENT_WEBSOCKET_TIMEOUT_S 10
 
@@ -188,7 +190,12 @@ static int __rrr_http_client_parse_config (
 
 		sprintf(array_tree_tmp, "%s;", array_definition);
 
-		if (rrr_array_tree_interpret_raw(&data->tree, array_tree_tmp, strlen(array_tree_tmp), "-") != 0 || data->tree == NULL) {
+		if (rrr_array_tree_interpret_raw (
+				&data->tree,
+				array_tree_tmp,
+				rrr_length_from_biglength_bug_const(strlen(array_tree_tmp)),
+				"-"
+		) != 0 || data->tree == NULL) {
 			RRR_MSG_0("Error while parsing array tree definition\n");
 			ret = 1;
 			goto out;
@@ -210,7 +217,7 @@ static int __rrr_http_client_parse_config (
 		goto out;
 	}
 
-	request_data->server= rrr_strdup(server);
+	request_data->server = rrr_strdup(server);
 	if (request_data->server == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_post_parse_config\n");
 		ret = 1;
@@ -273,34 +280,14 @@ static int __rrr_http_client_parse_config (
 	}
 
 	// HTTP port
-	const char *port = cmd_get_value(cmd, "port", 0);
-	uint64_t port_tmp = 0;
-	if (cmd_get_value (cmd, "port", 1) != NULL) {
-		RRR_MSG_0("Error: Only one 'port' argument may be specified\n");
-		ret = 1;
+	if ((ret = rrr_arguments_parse_port (
+			&request_data->http_port,
+			cmd,
+			"port",
+			request_data->transport_force == RRR_HTTP_TRANSPORT_HTTPS ? 443 : 80
+	)) != 0) {
 		goto out;
 	}
-	if (port != NULL) {
-		if (cmd_convert_uint64_10(port, &port_tmp)) {
-			RRR_MSG_0("Could not understand argument 'port', must be and unsigned integer\n");
-			ret = 1;
-			goto out;
-		}
-	}
-	if (port_tmp == 0) {
-		if (request_data->transport_force == RRR_HTTP_TRANSPORT_HTTPS) {
-			port_tmp = 443;
-		}
-		else {
-			port_tmp = 80;
-		}
-	}
-	if (port_tmp < 1 || port_tmp > 65535) {
-		RRR_MSG_0("HTTP port out of range (must be 1-65535, got %" PRIu64 ")\n", port_tmp);
-		ret = 1;
-		goto out;
-	}
-	request_data->http_port = port_tmp;
 
 	out:
 	RRR_FREE_IF_NOT_NULL(array_tree_tmp);
@@ -309,11 +296,11 @@ static int __rrr_http_client_parse_config (
 
 static int __rrr_http_client_final_write_callback (
 		const void *str,
-		rrr_length len,
+		rrr_nullsafe_len len,
 		void *arg
 ) {
 	ssize_t *bytes = arg;
-	*bytes = write (STDOUT_FILENO, str, len);
+	*bytes = write (STDOUT_FILENO, str, rrr_size_from_biglength_bug_const (len));
 	return 0;
 }
 
@@ -324,8 +311,8 @@ static int __rrr_http_client_final_callback (
 
 	int ret = 0;
 
-	rrr_length data_start = 0;
-	rrr_length data_size = rrr_nullsafe_str_len(response_data);
+	rrr_nullsafe_len data_start = 0;
+	rrr_nullsafe_len data_size = rrr_nullsafe_str_len(response_data);
 
 	if (data_size == 0) {
 		goto out;
@@ -350,7 +337,7 @@ static int __rrr_http_client_final_callback (
 		goto out;
 	}
 
-	RRR_MSG_2("Received %" PRIrrrl " bytes of data from HTTP library\n", data_size);
+	RRR_MSG_2("Received %" PRIrrr_nullsafe_len " bytes of data from HTTP library\n", data_size);
 
 	while (data_size > 0) {
 		ssize_t bytes = 0;
@@ -369,8 +356,11 @@ static int __rrr_http_client_final_callback (
 			goto out;
 		}
 		else {
-			data_start += bytes;
-			data_size -= bytes;
+			if ((rrr_nullsafe_len) bytes > data_size) {
+				RRR_BUG("BUG: Underflow in __rrr_http_client_final_callack\n");
+			}
+			data_start += (rrr_nullsafe_len) bytes;
+			data_size -= (rrr_nullsafe_len) bytes;
 		}
 	}
 
@@ -378,6 +368,19 @@ static int __rrr_http_client_final_callback (
 	return ret;
 }
 
+static int __rrr_http_client_failure_callback (
+		RRR_HTTP_CLIENT_FAILURE_CALLBACK_ARGS
+) {
+	struct rrr_http_client_data *http_client_data = arg;
+
+	(void)(transaction);
+	(void)(http_client_data);
+
+	RRR_MSG_0("Error while sending request: %s\n", error_msg);
+
+	return RRR_HTTP_SOFT_ERROR;
+}
+
 static int __rrr_http_client_unique_id_generator_callback (
 		RRR_HTTP_CLIENT_UNIQUE_ID_GENERATOR_CALLBACK_ARGS
 ) {
@@ -439,7 +442,7 @@ static int __rrr_http_client_redirect_callback (
 
 struct rrr_http_client_send_websocket_frame_callback_data {
 	void **data;
-	ssize_t *data_len;
+	rrr_biglength *data_len;
 	int *is_binary;
 	struct rrr_http_client_data *http_client_data;
 };
@@ -500,6 +503,7 @@ static int __rrr_http_client_send_websocket_frame_callback (RRR_HTTP_CLIENT_WEBS
 			0, // No ratelimit max bytes
 			1 * 1024 * 1024 * 1024, // 1 GB
 			__rrr_http_client_send_websocket_frame_final_callback,
+			NULL,
 			&callback_data
 	)) != 0) {
 		goto out;
@@ -507,13 +511,22 @@ static int __rrr_http_client_send_websocket_frame_callback (RRR_HTTP_CLIENT_WEBS
 
 	if (rrr_array_count(&array)) {
 		if (RRR_LL_COUNT(&http_client_data->tags)) {
-			ssize_t target_size = 0;
+			rrr_biglength target_size = 0;
 			int found_tags = 0;
 			if ((ret = rrr_array_selected_tags_export(&raw_tmp, &target_size, &found_tags, &array, &http_client_data->tags)) != 0) {
 				RRR_MSG_0("Failed to get specified array tags from input data, return was %i\n", ret);
 				goto out;
 			}
 
+			if (target_size > SSIZE_MAX) {
+				RRR_MSG_0("Exported size of array exceeds maximum (%llu > %lli)\n",
+					(unsigned long long) target_size,
+					(long long int) SSIZE_MAX
+				);
+				ret = RRR_HTTP_SOFT_ERROR;
+				goto out;
+			}
+
 			*data_len = target_size;
 
 			*data = raw_tmp;
@@ -521,7 +534,7 @@ static int __rrr_http_client_send_websocket_frame_callback (RRR_HTTP_CLIENT_WEBS
 			goto out;
 		}
 		else {
-			if ((ret = rrr_array_new_message_from_collection(&msg_tmp, &array, rrr_time_get_64(), NULL, 0)) != 0) {
+			if ((ret = rrr_array_new_message_from_array(&msg_tmp, &array, rrr_time_get_64(), NULL, 0)) != 0) {
 				RRR_MSG_0("Failed to create RRR array message from input data\n");
 				goto out;
 			}
@@ -542,7 +555,7 @@ static int __rrr_http_client_send_websocket_frame_callback (RRR_HTTP_CLIENT_WEBS
 	RRR_FREE_IF_NOT_NULL(msg_tmp);
 	RRR_FREE_IF_NOT_NULL(raw_tmp);
 	rrr_array_clear(&array);
-	if (ret != 0 && ret != RRR_SOCKET_READ_INCOMPLETE) {
+	if (ret != 0 && ret != RRR_SOCKET_READ_INCOMPLETE && EVENT_INITIALIZED(http_client_data->event_stdin)) {
 		EVENT_REMOVE(http_client_data->event_stdin);
 	}
 	return ret;
@@ -550,11 +563,11 @@ static int __rrr_http_client_send_websocket_frame_callback (RRR_HTTP_CLIENT_WEBS
 
 static int __rrr_http_client_receive_websocket_frame_nullsafe_callback (
 		const void *str,
-		rrr_length len,
+		rrr_nullsafe_len len,
 		void *arg
 ) {
 	(void)(arg);
-	ssize_t bytes = write (STDOUT_FILENO, str, len);
+	ssize_t bytes = write (STDOUT_FILENO, str, rrr_size_from_biglength_bug_const(len));
 	RRR_DBG_3("%lli bytes printed\n", (long long int) bytes);
 	return 0;
 }
@@ -571,7 +584,11 @@ static int __rrr_http_client_receive_websocket_frame_callback (RRR_HTTP_CLIENT_W
 		printf ("- (binary data) -\n");
 	}
 	else {
-		rrr_nullsafe_str_with_raw_do_const(payload, __rrr_http_client_receive_websocket_frame_nullsafe_callback, NULL);
+		rrr_nullsafe_str_with_raw_do_const (
+				payload,
+				__rrr_http_client_receive_websocket_frame_nullsafe_callback,
+				NULL
+		);
 	}
 
 	return 0;
@@ -618,7 +635,7 @@ static int rrr_http_client_event_periodic (RRR_EVENT_FUNCTION_PERIODIC_ARGS) {
 	}
 
 	if (rrr_http_client_active_transaction_count_get(data->http_client) == 0) {
-		if (!EVENT_PENDING(data->event_stdin)) {
+		if (!EVENT_INITIALIZED(data->event_stdin) || !EVENT_PENDING(data->event_stdin)) {
 			return RRR_EVENT_EXIT;
 		}
 	}
@@ -638,9 +655,15 @@ int main (int argc, const char **argv, const char **env) {
 
 	struct rrr_signal_handler *signal_handler = NULL;
 
-	if (rrr_log_init() != 0) {
+	if (rrr_allocator_init() != 0) {
+		ret = EXIT_FAILURE;
 		goto out_final;
 	}
+	if (rrr_log_init() != 0) {
+		ret = EXIT_FAILURE;
+		goto out_cleanup_allocator;
+	}
+
 	rrr_strerror_init();
 	rrr_signal_default_signal_actions_register();
 	signal_handler = rrr_signal_handler_push(rrr_signal_handler, NULL);
@@ -684,6 +707,8 @@ int main (int argc, const char **argv, const char **env) {
 	struct rrr_http_client_callbacks callbacks = {
 			__rrr_http_client_final_callback,
 			&data,
+			__rrr_http_client_failure_callback,
+			&data,
 			__rrr_http_client_redirect_callback,
 			&data,
 			__rrr_http_client_send_websocket_frame_callback,
@@ -775,7 +800,8 @@ int main (int argc, const char **argv, const char **env) {
 		cmd_destroy(&cmd);
 		rrr_socket_close_all();
 		rrr_strerror_cleanup();
-	out_final:
+	out_cleanup_allocator:
 		rrr_allocator_cleanup();
+	out_final:
 		return ret;
 }
