@@ -8,7 +8,6 @@ RRR_WITH_PERL5=yes
 RRR_WITH_NGHTTP2=yes
 RRR_WITH_MYSQL=yes
 RRR_WITH_OPENSSL=yes
-RRR_WITH_OPENSSL_CRYPT=
 RRR_WITH_LIBRESSL=no
 RRR_WITH_AUTO_SSL=yes
 
@@ -24,8 +23,8 @@ fi
 
 
 TEST="./test -e test_env.conf $1 $2 $3 $4"
-TEST_SOCKET=.rrr_test.sock
-TEST_DATA_FILE=.test_data.tmp
+TEST_SOCKET=/tmp/rrr-test.sock
+TEST_DATA_FILE=/tmp/rrr-test-data.tmp
 TEST_DATA_GENERATOR=./make_test_data
 TEST_DATA_ARRAY_DEFINITION=be4#int1,be3#int2,be2s#int3,be1#int4,sep1@1#sep1,le4@1#aaa,le3#bbb,le2s@1#ccc,le1#ddd,sep2#sep2,blob8@2#blob,msg#msg,str#emptystr,vain
 
@@ -36,7 +35,7 @@ SEND_IP=./send_ip
 SEND_IP_PORT=2222
 
 SEND_FIFO=./send_fifo
-SEND_FIFO_FILENAME=/tmp/rrr_test_fifo.sock
+SEND_FIFO_FILENAME=/tmp/rrr-test-fifo.sock
 
 SEND_WEBSOCKET="../rrr_http_client"
 SEND_WEBSOCKET_ARGS="-r /tmp -a $TEST_DATA_ARRAY_DEFINITION -s localhost -p 8880 -e /x -w"
@@ -132,16 +131,6 @@ do_test_fifo() {
 	fi
 }
 
-do_test_standalone() {
-	conf=$1
-	print_test_header STANDALONE $conf
-	echo \$ $TEST $conf
-	$TEST $conf
-	if test $? -ne 0; then
-		fail $conf 
-	fi
-}
-
 $TEST_DATA_GENERATOR $TEST_DATA_FILE
 if test $? -ne 0; then
 	fail "data generation"
@@ -153,11 +142,20 @@ do_test_simple test_dummy.conf
 do_test_simple test_averager.conf
 do_test_simple test_journal.conf
 
-do_test_standalone test_file.conf
+do_test_simple test_file.conf
 
+# Begin message DB tests
 rm -rf /tmp/rrr-test-msgdb*
-do_test_standalone test_incrementer.conf
+
+do_test_simple test_incrementer.conf
+
+do_test_simple test_cacher_1.conf 
+do_test_simple test_cacher_2.conf 
+do_test_simple test_cacher_3.conf 
+do_test_simple test_cacher_4.conf 
+
 rm -rf /tmp/rrr-test-msgdb* || exit 1
+# End message DB tests
 
 do_test_websocket test_websocket.conf
 
@@ -175,6 +173,7 @@ do_test_socket test_cmodule.conf
 echo "With perl5: $RRR_WITH_PERL5"
 if test "x$RRR_WITH_PERL5" != 'xno'; then
 	do_test_socket test_perl5.conf
+	do_test_simple test_mqtt_commands.conf
 fi
 
 echo "With python3: $RRR_WITH_PYTHON3"
