@@ -18,7 +18,7 @@ def config(config : rrr_config):
 
 	return True
 
-def source(socket: rrr_socket, message : rrr_msg_msg):
+def source(socket: rrr_socket, message : rrr_message):
 	global persistent_setting_a
 	global persistent_setting_b
 
@@ -28,18 +28,74 @@ def source(socket: rrr_socket, message : rrr_msg_msg):
 
 	return True
 
-def process(socket: rrr_socket, message: rrr_msg_msg):
+def process(socket: rrr_socket, message: rrr_message):
 	global persistent_setting_a
 	global persistent_setting_b
 
+	print("A")
+
 	if persistent_setting_b != "touched":
 		print("python3 configuration error, persistent_setting_b was not touched during config, value was '" + persistent_setting_b + "'")
 		return False
+	print("B")
+
+	##########################################
+	# METADATA
 
 	print ("python3 timestamp before: " + str(message.timestamp))
 	message.timestamp = message.timestamp + 1
 	print ("python3 timestamp after : " + str(message.timestamp))
 
+	message.set_topic("abcdef")
+	if message.get_topic() != "abcdef":
+		print("Python3 topic failure")
+		return False
+
+	##########################################
+	# IP
+	print("C")
+
+	message.set_ip("1.1.1.1", 3333)
+	message.set_ip_proto("uDp")
+	print("D")
+
+	if message.get_ip()[0] != "1.1.1.1":
+		print ("Python3 IP4 address failure")
+		return False
+	print("E")
+
+	if message.get_ip()[1] != 3333:
+		print ("Python3 IP4 port failure")
+		return False
+	print("F")
+
+	if message.get_ip_proto() != "UDP":
+		print ("Python3 IP protocol failure A")
+		return False
+
+	message.set_ip("::ffff:1.1.1.1", 3333)
+	message.set_ip_proto("tcp")
+
+	if message.get_ip()[0] != "::ffff:1.1.1.1":
+		print ("Python3 IP6 address failure")
+		return False
+
+	if message.get_ip()[1] != 3333:
+		print ("Python3 IP6 port failure")
+		return False
+
+	if message.get_ip_proto() != "TCP":
+		print ("Python3 IP protocol failureB")
+		return False
+
+	message.clear_ip()
+
+	if not message.get_ip() is None:
+		print ("Python3 IP clear failure")
+
+	##########################################
+	# ARRAY
+
 	if (not message.has_array()):
 		print ("python3 message did not have an array\n")
 		return False
@@ -73,6 +129,16 @@ def process(socket: rrr_socket, message: rrr_msg_msg):
 		for value in item:
 			print ("    " + str(value));
 
+	# Test for vain and empty string adding, leave them in the result array
+	vain = rrr_array_value()
+	vain.append(None)
+
+	strempty = rrr_array_value()
+	strempty.append("")
+
+	array_new.append(vain)
+	array_new.append(strempty)
+
 	message.set_array(array_new)
 
 	socket.send(message)
